
# coding: utf-8
"""
  GASSMANNAGER V5.0
 J.Coggins, Coggins Geosciences, Inc.
 
 Functions were tested by comparison to values in the fluid properties and
  Gassmann substitution sections of Mavko,'The Rock Physics Handbook'
  
  CONVERTED to PYTHON 3 on 16sep16
  
 

Revision Notes
 31aug16  Begin Gassmannager v1 using Fluid Designer as model
 31aug16  Completed mainmenu(),gettext()
 02sep16  Completed getfloat(), completed menu item 1
 04sep16  Added Regular expressions, completed menu items 2,f,3
 05sep16  Replaced statements with presskey() function
 06sep16  Added grain density to the input reservoir properties
 06sep16  Completed the Fluid Substitution for menu item 5
 06sep16  Completed Model File Save menu item s
 06sep16  Tested using parameters from:
          Smith,Sondergeld,Rai,'Gassmann fluid subtitutions: A tutorial,
          Geophysics,vol.68,no.2,pp.430-440
 07sep16  Completed model file load menu iterm         
          ***Need to Write Print Reservoir Model menu item
""" 
import math
#import numpy
import re
import matplotlib.pyplot as plt
import matplotlib.cm as cm
from pylab import *


#  set verbose mode for debuggins
verbose='yes'

#global variable definitions
press      = 0.0                #pressure in MPa
temp       = 0.0                #temperature in Celsius Degrees
vw         = 0.0                #velocity of water in meters/sec                   
rhow       = 0.0                #density of water in grams/cm^3
kfw        = 0.0                #fluid modulus of water in GPa
vb         = 0.0                #velocity of brine in meters/sec
rhob       = 0.0                #density of brine in grams/cm^3
kfb        = 0.0                #fluid modulus of brine in GPa
vo         = 0.0                #velocity of oil in meters/sec
rhoo       = 0.0                #density of oil in grams/cm^3
kfo        = 0.0                #fluid modulus of oil in GPa
vg         = 0.0                #velocity of gas in meters/sec
rhog       = 0.0                #density of gas in grams/cm^3
kfg        = 0.0                #fluid modulus of gs in GPa
sal        = 0.0                #salinity of brine in volume fraction
gasgrav    = 0.0                #gas gravity (unitless)
api        = 0.0                #api of oil (unitless)
gasgravo   = 0.0                #gas gravity in oil case (unitless)
rg         = 0.0                #gas oil ratio at STP (unitless)
fnamei     = ''                 #name of in-situ fluid
ftypei     = ''                 #type for in-situ fluid
vi         = 0.0
rhoi       = 0.0
kfi        = 0.0

kd         = 0.0                #dry rock moduls
k0         = 0.0                #grain modulus
fpfile     = ''                 #fluid property file name
fpfiledesc = ''                 #fluid file description
rmfile     = ''                 #reservoir model file name
rmfiledesc = ''                 #reservoir model file description
a=0.0
b=0.0
c=0.0
d=0.0
e=0.0
f=0.0
g=0.0
h=0.0
#variable for fluid mixes
nmix   = 0   #integer number of existing fluid mixes
fmname =list()      #text -list of fluid mix names
fmsat  =list()      #float
fmtype =list()      #text
fmvp   =list()      #float
fmk    =list()      #float
fmrho  =list()      #float

#variables for the Fluid Substitution Cases
nfsub   = 0   #integer number of existing fluid cases
fsname =list()      #text -list of fluid mix names
fssat  =list()      #float
fstype =list()      #text
fsvpr  =list()      #float
fskr   =list()      #float
fsrhor =list()      #float
fsvsr  =list()      #float
fsmur  =list()      #float

d=dict()

#define constants for h2oprop
#w=np.ndarray((5,4))

#Set up the Regular Expressions Functions
#get integer from string
numeric_const_pattern = r"""
[-+]? # optional sign
(?:
(?: \d+  ) # 1. 12. 123. etc 1 12 123 etc
)
"""
rk = re.compile(numeric_const_pattern, re.VERBOSE)
#print (rk.findall("-1 +1 2e9 +2E+09 -2e-9"))
#Get float from string
numeric_const_pattern = r"""
[-+]? # optional sign
(?:
(?: \d* \. \d+ ) # .1 .12 .123 etc 9.1 etc 98.1 etc
|
(?: \d+ \.? ) # 1. 12. 123. etc 1 12 123 etc
)
# followed by optional exponent part if desired
(?: [Ee] [+-]? \d+ ) ?
"""
rx = re.compile(numeric_const_pattern, re.VERBOSE)
#print (rx.findall("-1 +1 2e9 +2E+09 -2e-9"))
#get filename from string
numeric_const_pattern = r"""
^[A-Z,a-z]# first character must be a letter

(?:[A-Z,a-z,'_','-',0-9]+ )?
[.]?
(?:[A-Z,a-z]+ )?
"""
rfn = re.compile(numeric_const_pattern, re.VERBOSE)
#print (rfn.findall("file_name.txt"))


def presskey():
    answer=input('Press ENTER to Continue\n:')
    return True


def ask_ok(prompt, retries=4, complaint='Please Enter y(es) or n(o)'):
    while True:
        ok = input(prompt)
        if ok in ('y', 'ye', 'yes','Y','Yes','YES'):
            return True
        if ok in ('n', 'no', 'nop', 'nope','N','No','NO'):
            return False
        retries = retries - 1
        if retries < 0:
            return False
        print (complaint)

# Main Menu Function

def mainmenu():
    ansmm=''
    while ansmm!='q':
#        for i in range(0,50):
#            print('')
        print('/////////////////////////////////////////////////////////')        
        print('/           COGGINS GEOTOOLS GASSMANNAGER               /')
        print('/                                                       /')
        print('/                   By J. Coggins                       /')
        print('/              Coggins Geosciences, Inc.                /')
        print('/                                                       /')   
        print('/    GASSMANNAGER does Fluid Substitution               /')
        print('/   in Reservoir Rocks for Seismic Modeling             /')
        print('/                                                       /')
        print('/                Enter a Menu Choice:                   /')
        print('/    1    Set RESERVOIR ROCK PROPERTIES                 /')
        print('/    2    LOAD FLUID PROPERTIES from a FILE             /')
        print('/    f    PRINT FLUID PROPERTIES to Screen              /')
        print('/    3    Set IN-SITU FLUID PROPERTIES                  /')
        print('/    4    Calculate the DRY ROCK MODULUS                /')
        print('/    5    Calculate FLUID SUBSTITUTION PROPERTIES       /')
        print('/    p    PRINT the Reservoir MODEL to Screen           /')
        print('/    s    SAVE the RESERVOIR MODEL to a FILE            /')
        print('/    l    LOAD a RESERVOIR MODEL from a FILE            /')
        print('/    *    PLOT a Bar Chart of Reflection Coefficients   /')
        print('/    x    EXIT GASSMANNAGER                             /')
        print('/////////////////////////////////////////////////////////')       
        ansmm=input(' ENTER CHOICE: ')
        return ansmm

def loadfp(fname,fltype):
    
    #fltype is 'desc' (description)'water','brine','oil' or 'gas'
    
   
    fpfile=fname
     
    #print(' Input Fluid File    = ',fpfile)
    
    try:
        fhand=open(fpfile)
    except:
        print(' Input File: ',fpfile,' cannot be opened')
        presskey()
        fpfile=''
        return 0,0,0,0,0,0,0,0,
        
    #Read the Fluid Properties file
    d=dict()
    count=0
    for line in fhand:
#        print (line,end='')
        a=line.split()
        t=a[0]
        val=a[1]
        try:
            d[t]=float(val)
        except:
            d[t]=val
        #print(count,t,d[t]) 
        count=count+1
    #print (d)
    #print('')   
    if fltype=='desc':
        return d['desc'],0,0,0,0,0,0,0
    if fltype=='water':
        return d['press'],d['temp'],d['vw'],d['rhow'],d['kfw'],0,0,0
    if fltype=='brine':
        return d['press'],d['temp'],d['vb'],d['rhob'],d['kfb'],d['sal'],0,0
    if fltype=='oil':
        return (d['press'],d['temp'],d['vo'],d['rhoo'],d['kfo'],d['api'],
        d['gasgravo'],d['rg']) 
    if fltype=='gas':
        return d['press'],d['temp'],d['vg'],d['rhog'],d['kfg'],d['gasgrav'],0,0
    if fltype=='nfmix':
        zflag=0
        ii=0
        while zflag==0:
            try:
                zzz=d['fmix'+str(ii)]
                ii=ii+1
            except:
                nfm=ii
                zflag=1
        #print('zflag=',zflag,'nfmix=',nfm)
        return nfm,0,0,0,0,0,0,0
    if nfmix>0:
        #print('Entered nfmix>0 loop in load')
        for i in range(0,nfmix):
            if fltype=='fmix'+str(i):
                #print(fltype,i,'fmvp0=',d['fmvp0'],d['fmvp'+str(0)])
                return (d['fmix'+str(i)],d['fmname'+str(i)],d['fmtype'+str(i)],
                d['fmsat'+str(i)],d['fmvp'+str(i)],
                d['fmrho'+str(i)],d['fmk'+str(i)],0.0)
    
    
def printfp():
    print('')
    print ('  CURRENT FLUID PROPERTIES')
    if fname!='':
        print('Current Fluid Property File:',fname)
    if fpfiledesc!='':
        print('Current File Description:',fpfiledesc)
    print('')
    print(' Pressure   = ',press,' MPa')
    print(' Temp.      = ',temp,' Deg. C')
    print('')
    print('WATER')
    print(' vw         = ',vw,' m/s')
    print(' rhow       = ',rhow,' g/cm^3')
    print(' kfw        = ',kfw,' GPa')
    print('')
    print('BRINE')
    print(' Salinity   = ',sal,' Volume Fraction')   
    print(' vb         = ',vb,' m/s')
    print(' rhob       = ',rhob,' g/cm^3')
    print(' kfb        = ',kfb,' GPa')
    print('')
    print('OIL')
    print(' API        = ',api)   
    print(' gasgravo   = ',gasgravo)    
    print(' rg         = ',rg)      
    print(' vo         = ',vo,' m/s')
    print(' rhoo       = ',rhoo,' g/cm^3')
    print(' kfo        = ',kfo,' GPa')
    print('')
    print('GAS')
    print(' gasgrav    = ',gasgrav)   
    print(' vg         = ',vg,' m/s')
    print(' rhog       = ',rhog,' g/cm^3')
    print(' kfg        = ',kfg,' GPa')
    print('')
    if nfmix>0:
        print(' Mixed Fluid Cases')
        if nfmix > 0:
            for i in range(0,nfmix):
                print('Mix',i,fmname[i],': Sw=',fmsat[i],' HC Type=',
                fmtype[i])
                print(' vp         = ',fmvp[i],' m/s')
                print(' rho        = ',fmrho[i],' g/cm^3')
                print(' kf         = ',fmk[i],' GPa')
                      
    presskey()  
 
def saverm(fname):
    #Save the Reservoir Model to file
    # the file is identical to the fluid file, but includes the
    # reservoir properties and Gassmann Results
    print('')
    print(' Save Reservoir Model Properties File')
    print('')
        
    rmfile=fname
    
    print(' Reservoir Model File    = ',rmfile)
    print('')
    print('Enter a Model File Description (40 char max, NO SPACES)')
    ans=input(':')
    
    if len(ans)>40:
        rmfiledesc=ans[0:40]
    elif len(ans)==0:
        rmfiledesc='none' 
    else:
        rmfiledesc=ans
    try:
        fhand=open(rmfile,'w')
    except:
        print(' File: ',rmfile,' cannot be opened')
        presskey()
        rmfile=''
        rmfiledesc=''
        return
    #write the file description
    line0='desc '+rmfiledesc+'\n'
    fhand.write(line0)
    #write the fluid properties
    line1='press '+str(press)+' MPa\n'
    fhand.write(line1)
    line2='temp '+str(temp)+' DegC\n'
    fhand.write(line2)
    #
    line3='WATER PROPERTIES\n'
    fhand.write(line3)
    line4='vw '+str(vw)+' m/s\n'
    fhand.write(line4) 
    line5='rhow '+str(rhow)+' g/cm^3\n'
    fhand.write(line5)
    line6='kfw '+str(kfw)+' GPa\n'
    fhand.write(line6)
    #
    line7='BRINE PROPERTIES\n'
    fhand.write(line7)
    line8='sal '+str(sal)+' fraction\n'
    fhand.write(line8) 
    line9='vb '+str(vb)+' m/s\n'
    fhand.write(line9) 
    line10='rhob '+str(rhob)+' g/cm^3\n'
    fhand.write(line10)
    line11='kfb '+str(kfb)+' GPa\n'
    fhand.write(line11)
    #
    line12='OIL PROPERTIES\n'
    fhand.write(line12)
    line13='api '+str(api)+' unitless\n'
    fhand.write(line13) 
    line14='gasgravo '+str(gasgravo)+' unitless\n'
    fhand.write(line14) 
    line15='rg '+str(rg)+' unitless\n'
    fhand.write(line15) 
    line16='vo '+str(vo)+' m/s\n'
    fhand.write(line16) 
    line17='rhoo '+str(rhoo)+' g/cm^3\n'
    fhand.write(line17)
    line18='kfo '+str(kfo)+' GPa\n'
    fhand.write(line18)
    #
    line19='GAS PROPERTIES\n'
    fhand.write(line19)
    line20='gasgrav '+str(gasgrav)+' unitless\n'
    fhand.write(line20) 
    line21='vg '+str(vg)+' m/s\n'
    fhand.write(line21) 
    line22='rhog '+str(rhog)+' g/cm^3\n'
    fhand.write(line22)
    line23='kfg '+str(kfg)+' GPa\n'
    fhand.write(line23)
    #
    # write the fluid mixture data
    line=list()
    if nfmix>0:
        linea='FLUID MIX PROPERTIES\n'
        fhand.write(linea)
        for i in range(0,nfmix):
            line.append('fmix'+str(i)+' '+str(i)+'\n')
            line.append('fmname'+str(i)+' '+str(fmname[i])+'\n')
            line.append('fmtype'+str(i)+' '+str(fmtype[i])+'\n')
            line.append('fmsat'+str(i)+' '+str(fmsat[i])+' unitless\n')
            line.append('fmvp'+str(i)+' '+str(fmvp[i])+' m/s\n')
            line.append('fmrho'+str(i)+' '+str(fmrho[i])+' g/cm^3\n')
            line.append('fmk'+str(i)+' '+str(fmk[i])+' GPa\n')
            for j in range(0,7):
                fhand.write(line[i*7+j])

    #Write of fluid properties completed
    #Write the Reservoir Rock Properties
    linea='RESERVOIR ROCK PROPERTIES\n'
    fhand.write(linea)
    linea='rarea '+str(rarea)+'\n'
    fhand.write(linea)
    linea='rwell '+str(rwell)+'\n'
    fhand.write(linea)
    linea='rtype '+str(rtype)+'\n'
    fhand.write(linea)
    linea='rname '+str(rname)+'\n'
    fhand.write(linea)
    linea='rfluid '+str(rfluid)+'\n'
    fhand.write(linea)
    linea='rdepth '+str(rdepth)+'\n'
    fhand.write(linea)
    linea='vpr '+str(vpr)+' m/s\n'
    fhand.write(linea)
    linea='vsr '+str(vsr)+' m/s\n'
    fhand.write(linea)
    linea='rhor '+str(rhor)+' g/mc^3\n'
    fhand.write(linea)
    linea='phir '+str(phir)+' fraction\n'
    fhand.write(linea)
    linea='kr '+str(kr)+' GPa\n'
    fhand.write(linea)
    linea='mur '+str(mur)+' GPa\n'
    fhand.write(linea)
    linea='rhograin '+str(rhograin)+' g/cm^3\n'
    fhand.write(linea)
    #Write the In-Situ Fluid Properties
    linea='IN-SITU FLUID PROPERTIES\n'
    fhand.write(linea)
    linea='fnamei '+fnamei+'\n'
    fhand.write(linea)
    linea='ftypei '+ftypei+'\n'
    fhand.write(linea)
    linea='vi '+str(vi)+' m/s\n'
    fhand.write(linea)
    linea='rhoi '+str(rhoi)+' g/cm^3\n'
    fhand.write(linea)
    linea='kfi '+str(kfi)+' GPa\n'
    fhand.write(linea)
    #
    #Write the Dry rock modulus and Mineral Bulk Modulus
    linea='DRY ROCK AND MINERAL MODULI\n'
    fhand.write(linea)
    linea='k0 '+str(k0)+' GPa\n'
    fhand.write(linea)
    linea='kd '+str(kd)+' GPa\n'
    fhand.write(linea)
    #
    #
    linea='FLUID SUBSTITUTION CALCULATIONS\n'
    fhand.write(linea)
    #write number of fluid cases
    linea='nfsub '+str(nfsub)+'\n'
    fhand.write(linea)
    #write the Fluid substitution cases
    for i in range(0,nfsub):
        linea='case '+str(i)+'\n'
        fhand.write(linea)
        linea='fsname'+str(i)+' '+fsname[i]+'\n'
        fhand.write(linea)
        linea='fstype'+str(i)+' '+fstype[i]+'\n'
        fhand.write(linea)
        linea='fssat' +str(i)+' '+str(fssat[i])+' fraction\n'        
        fhand.write(linea)
        linea='fsvpr' +str(i)+' '+str(fsvpr[i])+' m/s\n'        
        fhand.write(linea)
        linea='fskr'  +str(i)+' '+str(fskr[i])+' GPa\n'        
        fhand.write(linea)
        linea='fsrhor'+str(i)+' '+str(fsrhor[i])+' g/cm^3\n'        
        fhand.write(linea)
        linea='fsvsr' +str(i)+' '+str(fsvsr[i])+' m/s\n'        
        fhand.write(linea)
        linea='fsmur' +str(i)+' '+str(fsmur[i])+' GPa\n'        
        fhand.write(linea)  
    fhand.close()    
    print('Save File Completed')
    presskey()    
    return  fpfiledesc       
      

def loadrm(fname,d):
    
    
    print('')
    print(' Loading Gassmann Model')
    print('')
   
    rmfile=fname
     
    try:
        fhand=open(rmfile)
    except:
        print(' Input File: ',rmfile,' cannot be opened')
        presskey()
        rmfile=''
        return 0,0,0,0,0,0,0,0,
        
    #Read the Properties file
    #d=dict()
    count=0
    for line in fhand:
#        print (line,end='')
        a=line.split()
        t=a[0]
        val=a[1]
        try:
            d[t]=float(val)
        except:
            d[t]=val
        #print(count,t,d[t]) 
        count=count+1
    #completed reading parameters into dictionary
    #print (d)
    print('')
    
    return d

def printrm():
    print('')
    prompt='Would you like to also list the Fluid Properties?'
    ans=ask_ok(prompt, retries=4, complaint='Please Enter y(es) or n(o)')
    print('')
    print ('  CURRENT GASSMANN MODEL')
    print('Current Model Property File:',rmfile)
    print('Current Model File Description:',rmfiledesc)
    if ans==True:
        print ('  Fluid Fluid Properties')
        print('Current Fluid File Description:',flfiledesc)
        print('')
        print(' Pressure   = ',press,' MPa')
        print(' Temp.      = ',temp,' Deg. C')
        print('')
        print('WATER')
        print(' vw         = ',vw,' m/s')
        print(' rhow       = ',rhow,' g/cm^3')
        print(' kfw        = ',kfw,' GPa')
        print('')
        print('BRINE')
        print(' Salinity   = ',sal,' Volume Fraction')   
        print(' vb         = ',vb,' m/s')
        print(' rhob       = ',rhob,' g/cm^3')
        print(' kfb        = ',kfb,' GPa')
        print('')
        print('OIL')
        print(' API        = ',api)   
        print(' gasgravo   = ',gasgravo)    
        print(' rg         = ',rg)      
        print(' vo         = ',vo,' m/s')
        print(' rhoo       = ',rhoo,' g/cm^3')
        print(' kfo        = ',kfo,' GPa')
        print('')
        print('GAS')
        print(' gasgrav    = ',gasgrav)   
        print(' vg         = ',vg,' m/s')
        print(' rhog       = ',rhog,' g/cm^3')
        print(' kfg        = ',kfg,' GPa')
        print('')
        if nfmix>0:
            print(' Mixed Fluid Cases')
            if nfmix > 0:
                for i in range(0,nfmix):
                    print('Mix',i,fmname[i],': Sw=',fmsat[i],' HC Type=',
                    fmtype[i])
                    print(' vp         = ',fmvp[i],' m/s')
                    print(' rho        = ',fmrho[i],' g/cm^3')
                    print(' kf         = ',fmk[i],' GPa')
                        
    #print the Gassmann Model Parameters  
    print('RESERVOIR ROCK PROPERTIES\n')
    print('rarea      =',rarea)
    print('rwell      =',rwell)
    print('rtype      =',rtype)
    print('rname      =',rname)
    print('rfluid     =',rfluid)
    print('rdepth     =',rdepth)
    print('vpr        =',vpr,' m/s')
    print('vsr        =',vsr,' m/s')
    print('rhor       =',rhor,' g/cm^3')
    print('phir       =',phir,' fraction')
    print('kr         =',kr,' GPa')
    print('mur        =',mur,' GPa')
    print('rhograin   =',rhograin,' g/cm^3')
    print('')
    print('IN-SITU FLUID PROPERTIES')
    print('fnamei     =',fnamei)
    print('ftypei     =',ftypei)
    print('vi         =',vi,' m/s')
    print('rhoi       =',rhoi,' g/cm^3')
    print('kfi        =',kfi,' GPa')
    print('')    
    print('DRY ROCK AND MINERAL MODULI')
    print('k0         =',k0,' GPa')
    print('kd         =',kd,' GPa')
    print('')
    print('FLUID SUBSTITUTION CALCULATIONS')
    if nfsub >0:
        for i in range(0,nfsub):
            print('Case ',i)
            print('fsname    =',fsname[i])
            print('fstype    =',fstype[i]) 
            print('fssat     =',fssat[i],' fraction')
            print('fsvpr     =',fsvpr[i],' m/s')
            print('fskr      =',fskr[i],' GPa')
            print('fsrhor    =',fsrhor[i],' g/cm^3')
            print('fsvsr     =',fsvsr[i],' m/s')
            print('fsmur     =',fsmur[i],' GPa')
   
    presskey()  


def getfname(rw,drect,ftype,tlab):
    #get a file name for read or write
    # and verify file existence   no default
    # rw is 'r' or 'w' for read or write, default is read
    # drect is a directory string (must end with / ) default is none
    # ftype is the file type to use / enforce  default is '.txt'
    #   is 
    print(' ',tlab)
    print('File type ',ftype,' will be appended to name if missing')
    if rw=='r':
        print(' Input file must already exist, or a null name is returned')
    
    ans=''
    fname=''
    while ans=='':
        ans=input('File Name: ')
    if ans[len(ans)-4:len(ans)-1]==ftype:
        fname=drect+ans
    else:
        fname=drect+ans+ftype
    try:
        fhand=open(fname,'r')
    except:
        if rw=='r':
            print(' Input File: ',fname,' cannot be opened')
            presskey()
            fname=''
            return fname
        elif rw=='w':
            return fname
    if rw=='w':
        print(' Output file ',fname,' exists')
        ans=ask_ok('Write over Existing File? y(es) or n(o)', retries=4, 
        complaint='Please Enter y(es) or n(o)')
        if ans=='False':
            fname=''
            fhand.close()
            return fname 
    fhand.close()
    return fname
 
def sat2dry(ks,kf,k0,phi):
    kd=0.0
    print('sat2dry ks,kf,k0,phi ',ks,kf,k0,phi)
    kd=(ks*(phi*k0/kf+1.0-phi)-k0)/(phi*k0/kf+ks/k0-1.0-phi)
    return kd
    
def dry2sat(kd,kf,ko,phi):
    ks=(phi*(1.0/ko-1/kf)+1.0/ko-1.0/kd) 
    ks=ks/((phi/kd)*(1.0/ko-1.0/kf)+(1.0/ko)*(1.0/ko-1.0/kd))
    return ks
    
def kflmix(k1,k2,sat1):
    kf=sat1/k1+(1.0-sat1)*k2
    kf=1.0/kf
    return kf
    
def rhofmix(rho1,rho2,sat1):
    rhof=rho1*sat1+rho2*(1.0-sat1)
    return rhof
 
def rhomix(rho,rhof,phi):
    rho=rhof*phi+rho*(1.0-phi)
    return rho 
    
def k2v(k,mu,rho):
    v=math.pow((k+4.0/3.0*mu)/rho*1.0e6,0.5)
    return v
    
def v2k(vp,vs,rho):
    mu=rho*vs*vs/1.0e6
    k=(vp*vp*rho)/1.0e6-4.0/3.0*mu
    return k
    
def v2mu(vs,rho):
    mu=vs*vs*rho/1.0e6
    return mu
    
def mu2v(mu,rho):
    v=math.pow(mu/rho*1.0e6,0.5)
    return v


    
def gettext(inname,maxlen,qtext1):
    """
    gettext returns a text string from the user
    inname is the name of the input property/variable (20 char max)'
    maxlen is max length of input string (def = 20)
    qtext1 is the secondary text  - default is '' 
    
    """
    inn=''
    if maxlen==0:
        maxl=20
    else:
        maxl=maxlen
    qt1=''
        # apply the default values for inputs
    if inname=='':
        print('Error: Input name inname cannot be null')
        return ''
    inn=inname[0:min(len(inname),20)]
    qt1=qtext1[0:min(len(qtext1),40)]    
    intext=''
    rflag=0
    while rflag==0:
        print(' Enter Text: ',inn.upper(),'(',maxl,' char. max):' ) 
        if qt1!='':
            print('',qt1)
        ans=''
        while ans=='':
            ans=input()
        intext=ans[0:min(len(ans),maxl)]
        print(inn,' = ',intext)
        ans=input('Enter y to accept,r to redo, q to quit: ') 
        print(' ')
        if ans=='r':
            rflag=0
        elif ans=='q':
            rflag=1
            print('Quit by user request')
            presskey()
            return 'q'
        elif ans =='y':
            rflag=1
        else:
            rflag=0
    return intext
    
def getfloat(inname,iunit,qtext1,tfmin,tfmax):
    """
    getfloat() returns a floating point value from the user
    inname is the name of the input property/variable (20 char max)
    iunit  is the units associated with value (10 char max) default-unitles
    qtext1 is the secondary text  - default is '' 
    tfmin is the minimum value accepted as text -default = 0.0
    tfmax is the maximum value accepted as text -default = 9.9e99
    """
    inn=''
    qt1=''
    iu=''
    fmin=0.0
    fmax=0.0
    # apply the default values for inputs
    if inname=='':
        print('Error: Input name inname cannot be null')
        return 0.0
    inn=inname[0:min(len(inname),20)]
    iu = iunit[0:min(len(iunit),10)]
    qt1=qtext1[0:min(len(qtext1),40)]
    if tfmin=='':
        fmin=0.0
    else:
        try:
            fmin=float(tfmin)
        except:
            fmin=0.0
    if tfmax=='':
        fmax=9.9e99
    else:
        try:
            fmax=float(tfmax)
        except:
            fmax=9.9e99
    infloat=0.0
    inval=0.0
    rflag=0
    while rflag==0:
        print(' Enter Value: ',inn.upper(),iu,' min=',fmin,' max=',fmax ) 
        if qt1!='':
            print('',qt1)
        ans=''
        while ans=='':
            ans=input()
        try:
            inval=float(ans)
        except:
            print('ERROR: Input Value is not in Floating Point format')
            print('Value set to 0.0')
            inval=0.0
        if inval<fmin:
            print('ERROR: Input is Greater that acceptable maximum')
            print('input value set to: ',fmin)
            infloat=fmin
        elif inval>fmax:
            print('ERROR: Input is less that acceptable minimum')
            print('input value set to: ',fmax)
            infloat=fmax
        else:
            infloat=inval
        print(inn,' = ',infloat,' ',iu)
        ans=input('Enter y to accept,r to redo, q to quit: ') 
        print('')
        if ans=='r':
            rflag=0
        elif ans=='q':
            rflag=1
            print('Quit by user request')
            presskey()
            return 0.0
        elif ans =='y':
            rflag=1
        else:
            rflag=0
    return infloat

def setisf(nfmix):
    #Choose the fluid to use as the in-situ case
    #the RESERVIOR ROCK PROPERTIES are taken to have this fluid 
    name=''
    ftype=''
    v=0.0
    rho=0.0
    k=0.0
    fchoice=0
    fch=0
    print('   SET THE IN-SITU FLUID CASE')
    print('')
    print(' Existing Fluid Cases ')
    print('0          Brine')
    print('1          Oil')
    print('2          Gas')
    if nfmix>0:
        for i in range(0,nfmix):
            print(str(i+3),'        ',fmname[i])
    iflag=0
    print('Enter the Number for the In-Situ Fluid Case')
    while iflag==0:
        ans=rk.findall(input(':'))  #ensure answer is an integer string
        try:
            fch=(int(ans[0]))           #findall returns a list
            if fch<0:
                fch=0
            elif fch>nfmix+2:
                fch=nfmix+2
            else:
                fchoice=fch
            iflag=1
        except:
            iflag=1
            print('Invalid Choice')
            presskey()
    print('Fluid Choice=',fchoice)
    if fchoice==0:
        name='Brine'
        ftype='brine'
        v=vb
        rho=rhob
        k=kfb
    elif fchoice==1:
        name='Oil'
        ftype='oil'
        v=vo
        rho=rhoo
        k=kfo
    elif fchoice==2:
        name='Gas'
        ftype='gas'
        v=vg
        rho=rhog
        k=kfg
    elif fchoice>2:
        name=fmname[fchoice-3]
        ftype=fmtype[fchoice-3]
        v=fmvp[fchoice-3]
        rho=fmrho[fchoice-3]
        k= fmk[fchoice-3]
    print('In-Situ Fluid:')
    print('Name=',name)
    print('Type=',ftype)
    print('v=',v)
    print('rho=',rho)
    print('k=',k)
    return name,ftype,v,rho,k
 
def barchart():
    """Plot a Barchart of Selected Reflection Coefficients
    """
    pflag=list()
    pcase=list()
    pflag=[0,0,0]
    
    pcase=['Brine','Oil','Gas']
    if nfmix>0:
        for i in range(nfmix):
            pflag.append(0)
            pcase.append(fmname[i])
    
    print('Bar Chart Plotting Tool') 
    print('')
    inname='Bounding Shale Density'
    iunit='g/cm^3'
    qtext1=''
    tfmin=1.0
    tfmax=4.0
    rhosh=getfloat(inname,iunit,qtext1,tfmin,tfmax)
    inname='Bounding Shale P-Wave Velocity'
    iunit='m/s'
    qtext1=''
    tfmin=1000.0
    tfmax=6000.0
    vpsh=getfloat(inname,iunit,qtext1,tfmin,tfmax)
    #select the Reservoir/Fluid Cases to Plot
    cflag=False
    while cflag==False:
        
        print('  Available Fluid Cases for Plotting')
        
        print('')
        print('0          Brine',end='')
        if pflag[0]>0:
            print('*')
        else:
            print('')
        print('1          Oil',end='')
        if pflag[1]>0:
            print('*')
        else:
            print('')
        print('2          Gas',end='')
        if pflag[2]>0:
            print('*')
        else:
            print('')
        if nfmix>0:
            for i in range(0,nfmix):
                print(str(i+3),'        ',fmname[i],end='')
                if pflag[i+3]>0:
                    print('*')
                else:
                    print('')   
        #set the cases to plot
           
        atext='Enter a Case to choose for Plotting, * to Make the Plot\n:'    
        ans=input(atext)
        if ans=='*':
            cflag=True
        else:
            try:
                choice=int(ans)
                if choice < (nfmix+3):
                    pflag[choice]=int(math.fabs(pflag[choice]-1))
            except:
                print('Please Enter a Case Number')
   
    names=list()
    values=list()
    colors=list()
    
    #build the data arrays
    numvals=0
    for i in range(0,nfmix+3):
        if pflag[i] > 0:
           names.append(pcase[i])
           rfc=(fsvpr[i]*fsrhor[i]-vpsh*rhosh)/(fsvpr[i]*fsrhor[i]+vpsh*rhosh)
           print('rfc ',i,rfc)
           values.append(rfc)
           jtxt=names[i]
           if 'bri' in jtxt.lower():
               colors.append('b')        
           elif 'oil' in jtxt.lower():
               colors.append('g')
           elif 'gas' in jtxt.lower():
               colors.append('r')
           else:
               colors.append('y')           
           numvals+=1
    #plotting
    ind = np.arange(numvals)  # the x locations for the groups
    width = 0.35       # the width of the bars
    
    fig, ax = plt.subplots()
    for i in range(0,numvals):
        ax.bar(ind[i]+width/2.0, values[i], width, color=colors[i])
    
    
    
    # add some text for labels, title and axes ticks
    ax.set_ylabel('Reflection Coefficient')
    ax.set_title('Refection Coefficients of Fluid Cases')
    ax.set_xticks(ind + width)
    ax.set_xticklabels(names)
    ax.yaxis.grid(b=True, which='major', color='b', linestyle='-')
    if values[0]<0.0:
        plt.ylim(max(0.0,max(values)),min(values))
    
    
    
    
    
    plt.show()       
    presskey()
        
    return
   
    
# Main Program Loop    
ansmm = ''
#==============================================================================
# testt=gettext('Well Name',20,'')
# print (testt)
#==============================================================================
while ansmm!='q':   
    ansmm = mainmenu()
    if verbose=='yes':
        print ('  Main Menu Choice =',ansmm)
    if ansmm=='1':                  #Set the RESERVOIR ROCK PROPERTIES
        print ('Set the RESERVOIR ROCK PROPERTIES')
        print('')
        rarea=gettext('Reservoir FIELD/AREA',20,'')
        rwell=gettext('Well Name',20,'')
        etext='(e.g. sand,unconsolidated sand,limestone)'
        rtype=gettext('Reservoir Type',20,etext)
        etext='(e.g. J-Sand, M3.3-A)'
        rname=gettext('Reservoir Name',20,etext)
        etext='(e.g. Brine,Gas Sw=20%,30 API Oil)'
        rfluid=gettext('In-Situ Reservoir Fluid',20,etext)
        etext='(e.g. -10200 ft tvdss,1520 m tvd)'
        rdepth=gettext('Reservoir Depth',20,etext)
        inname='P-Wave Velocity'
        iunit ='m/s'
        qtext1=''
        tfmin='100.0'
        tfmax='8000.0'
        vpr=getfloat(inname,iunit,qtext1,tfmin,tfmax)
        inname='S-Wave Velocity'
        iunit ='m/s'
        qtext1=''
        tfmin='100.0'
        tfmax='5000.0'
        vsr=getfloat(inname,iunit,qtext1,tfmin,tfmax)
        inname='Density'
        iunit ='g/cm^3'
        qtext1=''
        tfmin='1.0'
        tfmax='5.0'
        rhor=getfloat(inname,iunit,qtext1,tfmin,tfmax)
        inname='Porosity'
        iunit ='Percent(%)'
        qtext1=''
        tfmin='0.0'
        tfmax='100.0'
        phir=getfloat(inname,iunit,qtext1,tfmin,tfmax)
        phir=phir/100.0
        inname='Grain Density'
        iunit ='g/cm^3'
        qtext1='Quartz: 2.65 g/cm^3  Calcite: 2.71 g/cm^3'
        tfmin='1.0'
        tfmax='4.0'
        rhograin=getfloat(inname,iunit,qtext1,tfmin,tfmax)
        kr=v2k(vpr,vsr,rhor)
        mur=v2mu(vsr,rhor)
        #print('kr=',kr,' GPa')
        #print('mur=',mur,' GPa')
    if ansmm=='2':
        tlab='Enter a Fluid Properties Input File Name'
        rw='r'
        drect=''
        typ='.txt'
        fname=getfname(rw,drect,typ,tlab)
        print('INPUT FILE: ',fname)
        ftype='desc'
        fpfiledesc,d,e,f,g,a,b,c = loadfp(fname,ftype)
        #print('File Description: ',fpfiledesc)
        ftype='water'
        press,temp,vw,rhow,kfw,a,b,c = loadfp(fname,ftype)
        #print('water',press,temp,vw,rhow,kfw)
        ftype='brine'
        press,temp,vb,rhob,kfb,sal,a,b = loadfp(fname,ftype)
        #print('brine',press,temp,vb,rhob,kfb,sal)
        ftype='oil'
        press,temp,vo,rhoo,kfo,api,gasgravo,rg = loadfp(fname,ftype)
        #print('oil',press,temp,vo,rhoo,kfo,api,gasgravo,rg)
        ftype='gas'
        press,temp,vg,rhog,kfg,gasgrav,a,b = loadfp(fname,ftype)
        #print('gas',press,temp,vg,rhog,kfg,gasgrav)
        ftype='nfmix'
        nn,a,b,c,d,e,f,g=loadfp(fname,ftype)
        nfmix=int(nn)
        #print('nfmix=',nfmix)
        if nfmix>0:
            #clear the existing fluid mix lists
            del fmname[0:len(fmname)]
            del fmtype[0:len(fmtype)]
            del fmsat[0:len(fmsat)]
            del fmvp[0:len(fmvp)]
            del fmrho[0:len(fmrho)]
            del fmk[0:len(fmk)]
            for i in range(0,nfmix):
                ftype='fmix'+str(i)
                zzz,atext,btext,a,b,c,d,e=loadfp(fname,ftype)
                fmname.append(atext)
                fmtype.append(btext)
                fmsat.append(a)
                fmvp.append(b)
                fmrho.append(c)
                fmk.append(d)
                #print(zzz,atext,btext,a,b,c)
        print('Input of Fluid File Complete')
        presskey()
    if ansmm=='f':     #list the Fluid Properties to the screen
        printfp()
    if ansmm=='3':    #Set In-Situ Fluid Properties
        fnamei,ftypei,vi,rhoi,kfi=setisf(nfmix)
        print(fnamei,ftypei,vi,rhoi,kfi)
        presskey()
    if ansmm=='4':
        inname='Mineral Bulk Modulus'
        iunit ='GPa'
        qtext1='(Quartz:37 GPa Calcite:77 GPa'
        tfmin='10.0'
        tfmax='100.0'
        k0=getfloat(inname,iunit,qtext1,tfmin,tfmax)
        kd=sat2dry(kr,kfi,k0,phir)
        print('kr,kfi,k0,phir ',kr,kfi,k0,phir)
        print('Dry Bulk Modulus:',kd,' GPa')
        presskey()
    if ansmm=='5':            # Create Reservoir / Fluid Cases
        print(' FLUID SUBSTITUTION CALCULATION\n')
        #print('FLuid Cases')
        #print('0          Brine')
        #print('1          Oil')
        #print('2          Gas')
        
        nfsub=3+nfmix
        #clear out the existing lists
        del fsname[0:len(fsname)]    
        del fssat[0:len(fssat)] 
        del fstype[0:len(fstype)]    
        del fsvpr[0:len(fsvpr)]    
        del fsvsr[0:len(fsvsr)] 
        del fsrhor[0:len(fsrhor)]    
        del fskr[0:len(fskr)]
        del fsmur[0:len(fsmur)]
             
        f=0      #Brine Case
        fsname.append('Brine')
        fssat.append(100.0)
        fstype.append('brine')
        
        #calculate the bulk modulus
        fskr.append( dry2sat(kd,kfb,k0,phir))
        #Calculate the density
        fsrhor.append(rhomix(rhograin,rhob,phir))
        #set the Shear modulus
        fsmur.append(mur)    #shear modulus does not change
        #calculate the Vp
        fsvpr.append(k2v(fskr[f],fsmur[f],fsrhor[f]))
        #calculate Vs 
        fsvsr.append(mu2v(fsmur[f],fsrhor[f]))
        #print results
        print('fsname   ',str(f),' =',fsname[f])
        print('fssat    ',str(f),' =',fssat[f])
        print('fstype   ',str(f),' =',fstype[f])
        print('fsvpr    ',str(f),' =',fsvpr[f],' m/s')
        print('fsvsr    ',str(f),' =',fsvsr[f],' m/s')
        print('fsrhor   ',str(f),' =',fsrhor[f],' g/cm^3')
        print('fskr     ',str(f),' =',fskr[f],' GPa')
        print('fsmur    ',str(f),' =',fsmur[f],' GPa')
            
        f=1     #oil case
        fsname.append('Oil')
        fssat.append(0.0)
        fstype.append('oil')       
        #calculate the bulk modulus
        fskr.append( dry2sat(kd,kfo,k0,phir))
        #Calculate the density
        fsrhor.append(rhomix(rhograin,rhoo,phir))    
        #set the Shear modulus
        fsmur.append(mur)    #shear modulus does not change
        #calculate the Vp
        fsvpr.append(k2v(fskr[f],fsmur[f],fsrhor[f]))
        #calculate Vs 
        fsvsr.append(mu2v(fsmur[f],fsrhor[f]))
        #print results
        print('fsname   ',str(f),' =',fsname[f])
        print('fssat    ',str(f),' =',fssat[f])
        print('fstype   ',str(f),' =',fstype[f])
        print('fsvpr    ',str(f),' =',fsvpr[f],' m/s')
        print('fsvsr    ',str(f),' =',fsvsr[f],' m/s')
        print('fsrhor   ',str(f),' =',fsrhor[f],' g/cm^3')
        print('fskr     ',str(f),' =',fskr[f],' GPa')
        print('fsmur    ',str(f),' =',fsmur[f],' GPa')
        f=2     #Gas Case
        fsname.append('Gas')
        fssat.append(0.0)
        fstype.append('gas')        
        #calculate the bulk modulus
        fskr.append( dry2sat(kd,kfg,k0,phir))
        #Calculate the density
        fsrhor.append(rhomix(rhograin,rhog,phir))  
        #set the Shear modulus
        fsmur.append(mur)    #shear modulus does not change
        #calculate the Vp
        fsvpr.append(k2v(fskr[f],fsmur[f],fsrhor[f]))
        #calculate Vs 
        fsvsr.append(mu2v(fsmur[f],fsrhor[f]))
        #print results
        print('fsname   ',str(f),' =',fsname[f])
        print('fssat    ',str(f),' =',fssat[f])
        print('fstype   ',str(f),' =',fstype[f])
        print('fsvpr    ',str(f),' =',fsvpr[f],' m/s')
        print('fsvsr    ',str(f),' =',fsvsr[f],' m/s')
        print('fsrhor   ',str(f),' =',fsrhor[f],' g/cm^3')
        print('fskr     ',str(f),' =',fskr[f],' GPa')
        print('fsmur    ',str(f),' =',fsmur[f],' GPa')
        #Fluid Mix Cases
        if nfmix>0:
            for i in range(0,nfmix):
                f=3+i
                fsname.append(fmname[i])
                fssat.append(fmsat[i])
                fstype.append(fmtype[i])        
                #calculate the bulk modulus
                fskr.append( dry2sat(kd,fmk[i],k0,phir))
                #Calculate the density
                fsrhor.append(rhomix(rhograin,fmrho[i],phir))    
                #set the Shear modulus
                fsmur.append(mur)    #shear modulus does not change
                #calculate the Vp
                fsvpr.append(k2v(fskr[f],fsmur[f],fsrhor[f]))
                #calculate Vs 
                fsvsr.append(mu2v(fsmur[f],fsrhor[f]))
                #print results
                print('fsname   ',str(f),' =',fsname[f])
                print('fssat    ',str(f),' =',fssat[f])
                print('fstype   ',str(f),' =',fstype[f])
                print('fsvpr    ',str(f),' =',fsvpr[f],' m/s')
                print('fsvsr    ',str(f),' =',fsvsr[f],' m/s')
                print('fsrhor   ',str(f),' =',fsrhor[f],' g/cm^3')
                print('fskr     ',str(f),' =',fskr[f],' GPa')
                print('fsmur    ',str(f),' =',fsmur[f],' GPa')
        presskey()
    if ansmm=='p':
        printrm()  
    if ansmm=='s':
        tlab='Enter a Reservoir Properties Model Output File Name'
        rw='w'
        drect=''
        typ='.txt'
        fname=getfname(rw,drect,typ,tlab)
        rmfiledesc=saverm(fname)
    if ansmm=='l':
        print('Load the Gassmann Model File')
        tlab=''
        rw='r'
        drect=''
        typ='.txt'
        fname=getfname(rw,drect,typ,tlab)
        d=loadrm(fname,d)   #returns the directory with all the parameters
        print('After Return from loadrm')
        rmfile=fname
        rmfiledesc=d['desc']
        #optionally get fluid properties from the model file
        print('The Model File also contains the Fluid Properties')
        print('Reading the Fluid Properties will replace the existing ones')
        pstr='Would you like to read the Fluid Properties from the Model File?'
        ans=ask_ok(pstr, retries=4, complaint='Please Enter y(es) or n(o)')
        if ans==True: #real the fluid properties
            print('Reading the Fluid Properties...')
            flfile=fname
            flfiledesc=d['desc']
            press=d['press']
            temp =d['temp']
            #water
            vw        =d['vw']
            rhow      =d['rhow']
            kfw       =d['kfw']
            #brine
            sal       =d['sal']
            vb        =d['vb']
            rhob      =d['rhob']
            kfb       =d['kfb']
            #oil
            api       =d['api'] 
            gasgravo  =d['gasgravo'] 
            rg        =d['rg']
            vo        =d['vo']
            rhoo      =d['rhoo']
            kfo       =d['kfo']
            #gas
            gasgrav   =d['gasgrav'] 
            vg        =d['vg']
            rhog      =d['rhog']
            kfg       =d['kfg']
            #fluid Mixes
            #delete the values in the lists
            del fmname[0:len(fmname)]
            del fmtype[0:len(fmtype)]
            del fmsat[0:len(fmsat)]
            del fmvp[0:len(fmvp)]
            del fmrho[0:len(fmrho)]
            del fmk[0:len(fmk)]
            # determine nfmix
            kk=0
            nfflag=False
            while nfflag==False:
                try:
                    fm=d['fmix'+str(kk)]
                    kk=kk+1
                except:
                    nfflag=True
            nfmix=kk
            if nfmix>0:
                for i in range(0,nfmix):
                    fmname.append(d['fmname'+str(i)])
                    fmtype.append(d['fmtype'+str(i)])
                    fmsat.append(d['fmsat'+str(i)])
                    fmvp.append(d['fmvp'+str(i)])
                    fmrho.append(d['fmrho'+str(i)])
                    fmk.append(d['fmk'+str(i)])
            print('Read of Fluid Parameters Completed')
        #read the reservoir rock properties
        print('Read the Reservoir Rock Properties')
        rarea    =d['rarea']
        rwell    =d['rwell']
        rtype    =d['rtype']
        rname    =d['rname']
        rfluid   =d['rfluid']
        rdepth   =d['rdepth']
        vpr      =d['vpr']
        vsr      =d['vsr']
        rhor     =d['rhor']
        phir     =d['phir']
        kr       =d['kr']
        mur      =d['mur']
        rhograin =d['rhograin']
        #read the in-situ fluid properties
        print('Read the In-Situ Fluid Properties')
        fnamei =d['fnamei']
        ftypei  =d['ftypei']
        vi     =d['vi']
        rhoi   =d['rhoi']
        kfi    =d['kfi']
        #read the dry rock and mineral modulus
        print('Read the Dry Rock and Mineral Moduli')
        k0     =d['k0']
        kd     =d['kd']
        #read the fluid substitution calculations
        print('Read the Fluid Substitution Cases')
        nfsub  =int(d['nfsub'])
        if nfsub>0:
            del fsname[0:len(fsname)]    
            del fssat[0:len(fssat)] 
            del fstype[0:len(fstype)]    
            del fsvpr[0:len(fsvpr)]    
            del fsvsr[0:len(fsvsr)] 
            del fsrhor[0:len(fsrhor)]    
            del fskr[0:len(fskr)]
            del fsmur[0:len(fsmur)]
            for i in range(0,nfsub):
                fsname.append( d['fsname'+str(i)])
                fstype.append( d['fstype'+str(i)])
                fssat.append(  d['fssat' +str(i)])
                fsvpr.append(  d['fsvpr' +str(i)])
                fskr.append(   d['fskr'  +str(i)])
                fsrhor.append( d['fsrhor'+str(i)])
                fsvsr.append(  d['fsvsr' +str(i)])
                fsmur.append(  d['fsmur' +str(i)])
        
        
        presskey()
    if ansmm=='h':
        print('Help Not Available')
        presskey()
    if ansmm=='*':
        barchart()
    if ansmm=='x':
        print("EXIT: Enter 'y' to exit or anything else to continue" )
        print("Make sure you have saved your data to file!")
        ans=input()
        if ans=='y':
            ansmm='q'

print('  QUITTING PROGRAM by user request')

        



