
# coding: utf-8
"""
  OBLIQUE V4
  An AVO Modeling Tool written in Python 2.7 (Converted to 3.0)
  Oblique take the output Model file from GASSMANNAGER
  Bounding Shale properties are entered by user
  Makes AVA plots for selected fluids using a full Zoeppritz solution.
 J.Coggins, Coggins Geosciences, Inc.
 JLC@cogginsGeosciences.com
 www.CogginsGeosciences.com
 
Converted to Python3 on 16sep16

Revision Notes
 09sep16  Began Oblique V1 !! The basic system (PremiumBlend, Gassmannager,
          and now Oblique) for doing straight-forward seismic response modeling
          in a data room or other setting.
 09sep16  Completed and tested Menu Items 1,2,3
 10sep16  Finished implementing Menu Item 4
 11sep16  Checked saveava,completed and tested loadava and printava
          edited Oblique to check for existing parameters loaded for 
          an input file before prompting for new ones
 12sep16  Fixed the Zoepritz function and tested against the Gassmann
          Tutorial Paper
          Completed Plot Routine - works for only one bounding shale
          If more than one shale, need to prompt user
          Need to add the ava data export functionality
12sep16   Plot now prompts for shale case for multiple shales
""" 
 
import math
import numpy as np
import re
import matplotlib.pyplot as plt
#import matplotlib.cm as cm
#from pylab import *


#  set verbose mode for debuggins
verbose='yes'

#global variable definitions
press      = 0.0                #pressure in MPa
temp       = 0.0                #temperature in Celsius Degrees
vw         = 0.0                #velocity of water in meters/sec                   
rhow       = 0.0                #density of water in grams/cm^3
kfw        = 0.0                #fluid modulus of water in GPa
vb         = 0.0                #velocity of brine in meters/sec
rhob       = 0.0                #density of brine in grams/cm^3
kfb        = 0.0                #fluid modulus of brine in GPa
vo         = 0.0                #velocity of oil in meters/sec
rhoo       = 0.0                #density of oil in grams/cm^3
kfo        = 0.0                #fluid modulus of oil in GPa
vg         = 0.0                #velocity of gas in meters/sec
rhog       = 0.0                #density of gas in grams/cm^3
kfg        = 0.0                #fluid modulus of gs in GPa
sal        = 0.0                #salinity of brine in volume fraction
gasgrav    = 0.0                #gas gravity (unitless)
api        = 0.0                #api of oil (unitless)
gasgravo   = 0.0                #gas gravity in oil case (unitless)
rg         = 0.0                #gas oil ratio at STP (unitless)
fnamei     = ''                 #name of in-situ fluid
ftypei     = ''                 #type for in-situ fluid
vi         = 0.0
rhoi       = 0.0
kfi        = 0.0

kd         = 0.0                #dry rock moduls
k0         = 0.0                #grain modulus
fpfile     = ''                 #fluid property file name
fpfiledesc = ''                 #fluid file description
rmfile     = ''                 #reservoir model file name
rmfiledesc = ''                 #reservoir model file description
a=0.0
b=0.0
c=0.0
d=0.0
e=0.0
f=0.0
g=0.0
h=0.0
#variable for fluid mixes
nfmix   = 0   #integer number of existing fluid mixes
fmname =list()      #text -list of fluid mix names
fmsat  =list()      #float
fmtype =list()      #text
fmvp   =list()      #float
fmk    =list()      #float
fmrho  =list()      #float

#variables for the Fluid Substitution Cases
nfsub   = 0   #integer number of existing fluid cases
fsname =list()      #text -list of fluid mix names
fssat  =list()      #float
fstype =list()      #text
fsvpr  =list()      #float
fskr   =list()      #float
fsrhor =list()      #float
fsvsr  =list()      #float
fsmur  =list()      #float
nfsubmax=20

#Variables for Bounding Shale Values
maxsh=10
numsh  = 0.0
descsh = list()
vpsh   = list()
vssh   = list()
rhosh  = list()

pflag=list()
pcase=list()

#AVA Parameters
thetabeg=0.0
thetaend=0.0
thetainc=1.0
numtheta=91
ava=np.ndarray((maxsh,nfsubmax,numtheta),np.float)
ava.fill(0.0)
yplot=np.ndarray((numtheta),np.float)
yplot.fill(0.0)
avacase=list()
pcase=list()
pflag=list()

d=dict()

zflag=list()
zcase=list()
#define constants for h2oprop
#w=np.ndarray((5,4))

#Set up the Regular Expressions Functions
#get integer from string
numeric_const_pattern = r"""
[-+]? # optional sign
(?:
(?: \d+  ) # 1. 12. 123. etc 1 12 123 etc
)
"""
rk = re.compile(numeric_const_pattern, re.VERBOSE)
#print (rk.findall("-1 +1 2e9 +2E+09 -2e-9"))
#Get float from string
numeric_const_pattern = r"""
[-+]? # optional sign
(?:
(?: \d* \. \d+ ) # .1 .12 .123 etc 9.1 etc 98.1 etc
|
(?: \d+ \.? ) # 1. 12. 123. etc 1 12 123 etc
)
# followed by optional exponent part if desired
(?: [Ee] [+-]? \d+ ) ?
"""
rx = re.compile(numeric_const_pattern, re.VERBOSE)
#print (rx.findall("-1 +1 2e9 +2E+09 -2e-9"))
#get filename from string
numeric_const_pattern = r"""
^[A-Z,a-z]# first character must be a letter

(?:[A-Z,a-z,'_','-',0-9]+ )?
[.]?
(?:[A-Z,a-z]+ )?
"""
rfn = re.compile(numeric_const_pattern, re.VERBOSE)
#print (rfn.findall("file_name.txt"))


def presskey():
    answer=input('Press ENTER to Continue\n:')
    return True


def ask_ok(prompt, retries=4, complaint='Please Enter y(es) or n(o)'):
    while True:
        ok = input(prompt)
        if ok in ('y', 'ye', 'yes','Y','Yes','YES'):
            return True
        if ok in ('n', 'no', 'nop', 'nope','N','No','NO'):
            return False
        retries = retries - 1
        if retries < 0:
            return False
        print (complaint)

# Main Menu Function

def mainmenu():
    ansmm=''
    while ansmm!='q':
#        for i in range(0,50):
#            print('')
        print('/////////////////////////////////////////////////////////')        
        print('/           COGGINS GEOTOOLS OBLIQUE                    /')
        print('/        Amplitude Variation with Angle                 /')
        print('/                                                       /')
        print('/                   By J. Coggins                       /')
        print('/              Coggins Geosciences, Inc.                /')
        print('/             JLC@CogginsGeosciences.com                /')        
        print('/                                                       /')   
        print('/                                                       /')
        print('/                Enter a Menu Choice:                   /')
        print('/    l    LOAD a RESERVOIR/AVA MODEL from a FILE        /')
        print('/    2    SET the BOUNDING SHALE(S) PROPERTIES          /')
        print('/    3    SELECT the PORE FLUIDS TO MODEL               /')
        print('/    4    CALCULATE AMPLITUDE VARIATION WITH ANGLE      /')
        print('/    p    PRINT the RERSERVOIR/AVA MODEL to Screen      /')
        print('/    s    SAVE the RESERVOIR/AVA MODEL to a File        /')
        print('/    *    PLOT the AVA MODELS                           /')
        print('/    e    Export the AVA Data to a File                 /')       
        print('/    x    EXIT OBLIQUE AVA MODELER                      /')
        print('/////////////////////////////////////////////////////////')       
        ansmm=input(' ENTER CHOICE: ')
        return ansmm


 
def saveava(fname):
    #Save the Reservoir Model to file
    # the file is identical to the fluid file, but includes the
    # reservoir properties and Gassmann Results
    print('')
    print(' Save Reservoir Model Properties File')
    print('')
        
    rmfile=fname
    
    print(' Reservoir Model File    = ',rmfile)
    print('')
    print('Enter a Model File Description (40 char max, NO SPACES)')
    ans=input(':')
    
    if len(ans)>40:
        rmfiledesc=ans[0:40]
    elif len(ans)==0:
        rmfiledesc='none' 
    else:
        rmfiledesc=ans
    try:
        fhand=open(rmfile,'w')
    except:
        print(' File: ',rmfile,' cannot be opened')
        presskey()
        rmfile=''
        rmfiledesc=''
        return
    #write the file description
    line0='desc '+rmfiledesc+'\n'
    fhand.write(line0)
    #write the fluid properties
    line1='press '+str(press)+' MPa\n'
    fhand.write(line1)
    line2='temp '+str(temp)+' DegC\n'
    fhand.write(line2)
    #
    line3='WATER PROPERTIES\n'
    fhand.write(line3)
    line4='vw '+str(vw)+' m/s\n'
    fhand.write(line4) 
    line5='rhow '+str(rhow)+' g/cm^3\n'
    fhand.write(line5)
    line6='kfw '+str(kfw)+' GPa\n'
    fhand.write(line6)
    #
    line7='BRINE PROPERTIES\n'
    fhand.write(line7)
    line8='sal '+str(sal)+' fraction\n'
    fhand.write(line8) 
    line9='vb '+str(vb)+' m/s\n'
    fhand.write(line9) 
    line10='rhob '+str(rhob)+' g/cm^3\n'
    fhand.write(line10)
    line11='kfb '+str(kfb)+' GPa\n'
    fhand.write(line11)
    #
    line12='OIL PROPERTIES\n'
    fhand.write(line12)
    line13='api '+str(api)+' unitless\n'
    fhand.write(line13) 
    line14='gasgravo '+str(gasgravo)+' unitless\n'
    fhand.write(line14) 
    line15='rg '+str(rg)+' unitless\n'
    fhand.write(line15) 
    line16='vo '+str(vo)+' m/s\n'
    fhand.write(line16) 
    line17='rhoo '+str(rhoo)+' g/cm^3\n'
    fhand.write(line17)
    line18='kfo '+str(kfo)+' GPa\n'
    fhand.write(line18)
    #
    line19='GAS PROPERTIES\n'
    fhand.write(line19)
    line20='gasgrav '+str(gasgrav)+' unitless\n'
    fhand.write(line20) 
    line21='vg '+str(vg)+' m/s\n'
    fhand.write(line21) 
    line22='rhog '+str(rhog)+' g/cm^3\n'
    fhand.write(line22)
    line23='kfg '+str(kfg)+' GPa\n'
    fhand.write(line23)
    #
    # write the fluid mixture data
    line=list()
    if nfmix>0:
        linea='FLUID MIX PROPERTIES\n'
        fhand.write(linea)
        for i in range(0,nfmix):
            line.append('fmix'+str(i)+' '+str(i)+'\n')
            line.append('fmname'+str(i)+' '+str(fmname[i])+'\n')
            line.append('fmtype'+str(i)+' '+str(fmtype[i])+'\n')
            line.append('fmsat'+str(i)+' '+str(fmsat[i])+' unitless\n')
            line.append('fmvp'+str(i)+' '+str(fmvp[i])+' m/s\n')
            line.append('fmrho'+str(i)+' '+str(fmrho[i])+' g/cm^3\n')
            line.append('fmk'+str(i)+' '+str(fmk[i])+' GPa\n')
            for j in range(0,7):
                fhand.write(line[i*7+j])

    #Write of fluid properties completed
    #Write the Reservoir Rock Properties
    linea='RESERVOIR ROCK PROPERTIES\n'
    fhand.write(linea)
    linea='rarea '+str(rarea)+'\n'
    fhand.write(linea)
    linea='rwell '+str(rwell)+'\n'
    fhand.write(linea)
    linea='rtype '+str(rtype)+'\n'
    fhand.write(linea)
    linea='rname '+str(rname)+'\n'
    fhand.write(linea)
    linea='rfluid '+str(rfluid)+'\n'
    fhand.write(linea)
    linea='rdepth '+str(rdepth)+'\n'
    fhand.write(linea)
    linea='vpr '+str(vpr)+' m/s\n'
    fhand.write(linea)
    linea='vsr '+str(vsr)+' m/s\n'
    fhand.write(linea)
    linea='rhor '+str(rhor)+' g/mc^3\n'
    fhand.write(linea)
    linea='phir '+str(phir)+' fraction\n'
    fhand.write(linea)
    linea='kr '+str(kr)+' GPa\n'
    fhand.write(linea)
    linea='mur '+str(mur)+' GPa\n'
    fhand.write(linea)
    linea='rhograin '+str(rhograin)+' g/cm^3'
    fhand.write(linea)
    #Write the In-Situ Fluid Properties
    linea='IN-SITU FLUID PROPERTIES\n'
    fhand.write(linea)
    linea='fnamei '+fnamei+'\n'
    fhand.write(linea)
    linea='ftypei '+ftypei+'\n'
    fhand.write(linea)
    linea='vi '+str(vi)+' m/s\n'
    fhand.write(linea)
    linea='rhoi '+str(rhoi)+' g/cm^3\n'
    fhand.write(linea)
    linea='kfi '+str(kfi)+' GPa\n'
    fhand.write(linea)
    #
    #Write the Dry rock modulus and Mineral Bulk Modulus
    linea='DRY ROCK AND MINERAL MODULI\n'
    fhand.write(linea)
    linea='k0 '+str(k0)+' GPa\n'
    fhand.write(linea)
    linea='kd '+str(kd)+' GPa\n'
    fhand.write(linea)
    #
    #
    linea='FLUID SUBSTITUTION CALCULATIONS\n'
    fhand.write(linea)
    #write number of fluid cases
    linea='nfsub '+str(nfsub)+'\n'
    fhand.write(linea)
    #write the Fluid substitution cases
    for i in range(0,nfsub):
        linea='case '+str(i)+'\n'
        fhand.write(linea)
        linea='fsname'+str(i)+' '+fsname[i]+'\n'
        fhand.write(linea)
        linea='fstype'+str(i)+' '+fstype[i]+'\n'
        fhand.write(linea)
        linea='fssat' +str(i)+' '+str(fssat[i])+' fraction\n'        
        fhand.write(linea)
        linea='fsvpr' +str(i)+' '+str(fsvpr[i])+' m/s\n'        
        fhand.write(linea)
        linea='fskr'  +str(i)+' '+str(fskr[i])+' GPa\n'        
        fhand.write(linea)
        linea='fsrhor'+str(i)+' '+str(fsrhor[i])+' g/cm^3\n'        
        fhand.write(linea)
        linea='fsvsr' +str(i)+' '+str(fsvsr[i])+' m/s\n'        
        fhand.write(linea)
        linea='fsmur' +str(i)+' '+str(fsmur[i])+' GPa\n'        
        fhand.write(linea)  
#    numsh  = 0.0
#    descsh = list()
#    vpsh   = list()
#    vssh   = list()
#    rhosh  = list()     
    #print the bounding shale data
    if numsh>0:
        linea='BOUNDING SHALE PROPERTIES\n'
        fhand.write(linea)    
        linea='numsh '+str(numsh)+'\n'
        fhand.write(linea) 
        for i in range(0,numsh):
            linea='descsh'+str(i)+' '+descsh[i]+'\n'
            fhand.write(linea)
            linea='vpsh'+str(i)+' '+str(vpsh[i])+' m/s\n'
            fhand.write(linea)
            linea='vssh'+str(i)+' '+str(vssh[i])+' m/s\n'
            fhand.write(linea)
            linea='rhosh'+str(i)+' '+str(rhosh[i])+' g/cm^3\n'
            fhand.write(linea)
    #    pflag=list()
    #    pcase=list()
        #print the modeled ava fluid cases
        linea='FLUID CASES SELECTED FOR AVA\n'
        fhand.write(linea)
        linea='nfsub '+str(nfsub)+'\n'
        fhand.write(linea) 
        for i in range(0,nfsub):
            linea='pcase'+str(i)+' '+pcase[i]+'\n'
            fhand.write(linea)
            linea='pflag'+str(i)+' '+str(pflag[i])+' 1=selected\n'
            fhand.write(linea)
        linea='AVA ANGLE RANGES\n' 
        fhand.write(linea)
        linea='thetabeg '+str(thetabeg)+' Degrees\n'
        fhand.write(linea)  
        linea='thetaend '+str(thetaend)+' Degrees\n'
        fhand.write(linea)   
        linea='thetainc '+str(thetainc)+' Degrees\n'
        fhand.write(linea)                
    fhand.close()    
    print('Save File Completed')
    presskey()    
    return       
      

def loadava(fname,d):
    
    
    print('')
    print(' Loading Model')
    print('')
   
    rmfile=fname
     
    try:
        fhand=open(rmfile)
    except:
        print(' Input File: ',rmfile,' cannot be opened')
        presskey()
        rmfile=''
        return 0,0,0,0,0,0,0,0,
        
    #Read the Properties file
    #d=dict()
    count=0
    for line in fhand:
#        print (line,end='')
        a=line.split()
        t=a[0]
        val=a[1]
        try:
            d[t]=float(val)
        except:
            d[t]=val
        #print(count,t,d[t]) 
        count=count+1
    #completed reading parameters into dictionary
    #print (d)
    print('')
    
    return d

def printava():
    
    ans=True
    print('')
    print ('  CURRENT GASSMANN MODEL')
    print('Current Model Property File:',rmfile)
    print('Current Model File Description:',rmfiledesc)
    if ans==True:
        print ('  Fluid Fluid Properties')
        print('Current Fluid File Description:',flfiledesc)
        print('')
        print(' Pressure   = ',press,' MPa')
        print(' Temp.      = ',temp,' Deg. C')
        print('')
        print('WATER')
        print(' vw         = ',vw,' m/s')
        print(' rhow       = ',rhow,' g/cm^3')
        print(' kfw        = ',kfw,' GPa')
        print('')
        print('BRINE')
        print(' Salinity   = ',sal,' Volume Fraction')   
        print(' vb         = ',vb,' m/s')
        print(' rhob       = ',rhob,' g/cm^3')
        print(' kfb        = ',kfb,' GPa')
        print('')
        print('OIL')
        print(' API        = ',api)   
        print(' gasgravo   = ',gasgravo)    
        print(' rg         = ',rg)      
        print(' vo         = ',vo,' m/s')
        print(' rhoo       = ',rhoo,' g/cm^3')
        print(' kfo        = ',kfo,' GPa')
        print('')
        print('GAS')
        print(' gasgrav    = ',gasgrav)   
        print(' vg         = ',vg,' m/s')
        print(' rhog       = ',rhog,' g/cm^3')
        print(' kfg        = ',kfg,' GPa')
        print('')
        if nfmix>0:
            print(' Mixed Fluid Cases')
            if nfmix > 0:
                for i in range(0,nfmix):
                    print('Mix',i,fmname[i],': Sw=',fmsat[i],' HC Type=',
                    fmtype[i])
                    print(' vp         = ',fmvp[i],' m/s')
                    print(' rho        = ',fmrho[i],' g/cm^3')
                    print(' kf         = ',fmk[i],' GPa')
                        
    #print the Gassmann Model Parameters  
    print('RESERVOIR ROCK PROPERTIES\n')
    print('rarea      =',rarea)
    print('rwell      =',rwell)
    print('rtype      =',rtype)
    print('rname      =',rname)
    print('rfluid     =',rfluid)
    print('rdepth     =',rdepth)
    print('vpr        =',vpr,' m/s')
    print('vsr        =',vsr,' m/s')
    print('rhor       =',rhor,' g/cm^3')
    print('phir       =',phir,' fraction')
    print('kr         =',kr,' GPa')
    print('mur        =',mur,' GPa')
    print('rhograin   =',rhograin,' g/cm^3')
    print('')
    print('IN-SITU FLUID PROPERTIES')
    print('fnamei     =',fnamei)
    print('ftypei     =',ftypei)
    print('vi         =',vi,' m/s')
    print('rhoi       =',rhoi,' g/cm^3')
    print('kfi        =',kfi,' GPa')
    print('')    
    print('DRY ROCK AND MINERAL MODULI')
    print('k0         =',k0,' GPa')
    print('kd         =',kd,' GPa')
    print('')
    print('FLUID SUBSTITUTION CALCULATIONS')
    if nfsub >0:
        for i in range(0,nfsub):
            print('Case ',i)
            print('fsname    =',fsname[i])
            print('fstype    =',fstype[i]) 
            print('fssat     =',fssat[i],' fraction')
            print('fsvpr     =',fsvpr[i],' m/s')
            print('fskr      =',fskr[i],' GPa')
            print('fsrhor    =',fsrhor[i],' g/cm^3')
            print('fsvsr     =',fsvsr[i],' m/s')
            print('fsmur     =',fsmur[i],' GPa')
     #print the bounding shale data
    
    
    if numsh>0:
        print('BOUNDING SHALE PROPERTIES')
        print('Number of Shales: ',numsh)
        for i in range(0,int(numsh)): 
            print('Case ',i)
            print('descsh    =',descsh[i])
            print('vpsh      =',vpsh[i],' m/s')
            print('vssh      =',vssh[i],' m/s')
            print('rhosh     =',rhosh[i],' g/cm^3')
        print('FLUID CASES SELECTED FOR AVA')
        if nfsub>0:
            for i in range(0,nfsub):
                print('Case       =',pcase[i])                 
                print('Flag      =',pflag[i])
        print('AVA ANGLE PARAMETERS')
        print('thetabeg    =',thetabeg,' Degrees')
        print('thetaend    =',thetaend,' Degrees')
        print('thetainc    =',thetainc,' Degrees') 
    print('\nLISTING COMPLETED')
    presskey()  


def getfname(rw,drect,ftype,tlab):
    #get a file name for read or write
    # and verify file existence   no default
    # rw is 'r' or 'w' for read or write, default is read
    # drect is a directory string (must end with / ) default is none
    # ftype is the file type to use / enforce  default is '.txt'
    #   is 
    print(' ',tlab)
    print('File type ',ftype,' will be appended to name if missing')
    if rw=='r':
        print(' Input file must already exist, or a null name is returned')
    
    ans=''
    fname=''
    while ans=='':
        ans=input('File Name: ')
    if ans[len(ans)-4:len(ans)-1]==ftype:
        fname=drect+ans
    else:
        fname=drect+ans+ftype
    try:
        fhand=open(fname,'r')
    except:
        if rw=='r':
            print(' Input File: ',fname,' cannot be opened')
            presskey()
            fname=''
            return fname
        elif rw=='w':
            return fname
    if rw=='w':
        print(' Output file ',fname,' exists')
        ans=ask_ok('Write over Existing File? y(es) or n(o)', retries=4, 
        complaint='Please Enter y(es) or n(o)')
        if ans=='False':
            fname=''
            fhand.close()
            return fname 
    fhand.close()
    return fname

def zoepj(vp1,vs1,rho1,vp2,vs2,rho2,theta1):
# angles in degrees
# velocities in m/s
# densities in g/cm3
    tp1=np.radians(theta1)
    p1=rho1
    p2=rho2
# calculate ray parameter and angles    
    p=np.sin(tp1)/vp1
    tp2=np.arcsin(p*vp2)
    ts1=np.arcsin(p*vs1)
    ts2=np.arcsin(p*vs2)
    
# calculate parameters
    a=p2*(1-2*np.sin(ts2)**2)-p1*(1-2*np.sin(ts1)**2)
    b=p2*(1-2*np.sin(ts2)**2)+2*p1*np.sin(ts1)**2
    c=p1*(1-2*np.sin(ts1)**2)+2*p2*np.sin(ts2)**2
    d=2*(p2*vs2**2-p1*vs1**2)
    E=b*np.cos(tp1)/vp1+c*np.cos(tp2)/vp2
    F=b*np.cos(ts1)/vs1+c*np.cos(ts2)/vs2
    G=a-d*np.cos(tp1)/vp1*np.cos(ts2)/vs2
    H=a-d*np.cos(tp2)/vp2*np.cos(ts1)/vs1
    D=E*F+G*H*p**2
    
# calculate rpp   
    rpp=1/D*(F*(b*np.cos(tp1)/vp1-c*np.cos(tp2)/vp2)-(a+d*np.cos(tp1)/vp1* \
          np.cos(ts2)/vs2)*H*p**2)
    return rpp

def akirich(vp1,vs1,rho1,vp2,vs2,rho2,theta1):
# angles in radians
# velocities in m/s
# densities in g/cm3
    tp1=theta1
    p1=rho1
    p2=rho2
    p=math.sin(theta1)/vp1
    vsa=(vs1+vs2)/2.0
    pa=(p1+p2)/2.0
    vpa=(vp1+vp2)/2.0
    dp=p2-p1
    dvp=vp2-vp1
    dvs=vs2-vs1
    rpp=0.5*(1-4*p*p*vsa*vsa)*dp/pa
    rpp=rpp+1/(2*math.pow(math.cos(tp1),2))*dvp/vpa
    rpp=rpp-4*p*p*vsa*vsa*dvs/vsa
    return rpp

    
def gettext(inname,maxlen,qtext1):
    """
    gettext returns a text string from the user
    inname is the name of the input property/variable (20 char max)'
    maxlen is max length of input string (def = 20)
    qtext1 is the secondary text  - default is '' 
    
    """
    inn=''
    if maxlen==0:
        maxl=20
    else:
        maxl=maxlen
    qt1=''
        # apply the default values for inputs
    if inname=='':
        print('Error: Input name inname cannot be null')
        return ''
    inn=inname[0:min(len(inname),20)]
    qt1=qtext1[0:min(len(qtext1),40)]    
    intext=''
    rflag=0
    while rflag==0:
        print(' Enter Text: ',inn.upper(),'(',maxl,' char. max):' ) 
        if qt1!='':
            print('',qt1)
        ans=''
        while ans=='':
            ans=input()
        intext=ans[0:min(len(ans),maxl)]
        print(inn,' = ',intext)
        ans=input('Enter y to accept,r to redo, q to quit: ') 
        print(' ')
        if ans=='r':
            rflag=0
        elif ans=='q':
            rflag=1
            print('Quit by user request')
            presskey()
            return 'q'
        elif ans =='y':
            rflag=1
        else:
            rflag=0
    return intext
    
def getfloat(inname,iunit,qtext1,tfmin,tfmax):
    """
    getfloat() returns a floating point value from the user
    inname is the name of the input property/variable (20 char max)
    iunit  is the units associated with value (10 char max) default-unitles
    qtext1 is the secondary text  - default is '' 
    tfmin is the minimum value accepted as text -default = 0.0
    tfmax is the maximum value accepted as text -default = 9.9e99
    """
    inn=''
    qt1=''
    iu=''
    fmin=0.0
    fmax=0.0
    # apply the default values for inputs
    if inname=='':
        print('Error: Input name inname cannot be null')
        return 0.0
    inn=inname[0:min(len(inname),20)]
    iu = iunit[0:min(len(iunit),10)]
    qt1=qtext1[0:min(len(qtext1),40)]
    if tfmin=='':
        fmin=0.0
    else:
        try:
            fmin=float(tfmin)
        except:
            fmin=0.0
    if tfmax=='':
        fmax=9.9e99
    else:
        try:
            fmax=float(tfmax)
        except:
            fmax=9.9e99
    infloat=0.0
    inval=0.0
    rflag=0
    while rflag==0:
        print(' Enter Value: ',inn.upper(),iu,' min=',fmin,' max=',fmax ) 
        if qt1!='':
            print('',qt1)
        ans=''
        while ans=='':
            ans=input()
        try:
            inval=float(ans)
        except:
            print('ERROR: Input Value is not in Floating Point format')
            print('Value set to 0.0')
            inval=0.0
        if inval<fmin:
            print('ERROR: Input is Greater that acceptable maximum')
            print('input value set to: ',fmin)
            infloat=fmin
        elif inval>fmax:
            print('ERROR: Input is less that acceptable minimum')
            print('input value set to: ',fmax)
            infloat=fmax
        else:
            infloat=inval
        print(inn,' = ',infloat,' ',iu)
        ans=input('Enter y to accept,r to redo, q to quit: ') 
        print('')
        if ans=='r':
            rflag=0
        elif ans=='q':
            rflag=1
            print('Quit by user request')
            presskey()
            return 0.0
        elif ans =='y':
            rflag=1
        else:
            rflag=0
    return infloat

def setisf(nfmix):
    #Choose the fluid to use as the in-situ case
    #the RESERVIOR ROCK PROPERTIES are taken to have this fluid 
    name=''
    ftype=''
    v=0.0
    rho=0.0
    k=0.0
    fchoice=0
    fch=0
    print('   SET THE IN-SITU FLUID CASE')
    print('')
    print(' Existing Fluid Cases ')
    print('0          Brine')
    print('1          Oil')
    print('2          Gas')
    if nfmix>0:
        for i in range(0,nfmix):
            print(str(i+3),'        ',fmname[i])
    iflag=0
    print('Enter the Number for the In-Situ Fluid Case')
    while iflag==0:
        ans=rk.findall(input(':'))  #ensure answer is an integer string
        try:
            fch=(int(ans[0]))           #findall returns a list
            if fch<0:
                fch=0
            elif fch>nfmix+2:
                fch=nfmix+2
            else:
                fchoice=fch
            iflag=1
        except:
            iflag=1
            print('Invalid Choice')
            presskey()
    print('Fluid Choice=',fchoice)
    if fchoice==0:
        name='Brine'
        ftype='brine'
        v=vb
        rho=rhob
        k=kfb
    elif fchoice==1:
        name='Oil'
        ftype='oil'
        v=vo
        rho=rhoo
        k=kfo
    elif fchoice==2:
        name='Gas'
        ftype='gas'
        v=vg
        rho=rhog
        k=kfg
    elif fchoice>2:
        name=fmname[fchoice-3]
        ftype=fmtype[fchoice-3]
        v=fmvp[fchoice-3]
        rho=fmrho[fchoice-3]
        k= fmk[fchoice-3]
    print('In-Situ Fluid:')
    print('Name=',name)
    print('Type=',ftype)
    print('v=',v)
    print('rho=',rho)
    print('k=',k)
    return name,ftype,v,rho,k
 
def plotavo():
    """Plot The AVO
    """
    #print('print AVA\n',ava)
    x=list()
    y=list()

    zflag=list()
    zcase=list()
    zflag=[0,0,0]
    
    zcase=['Brine','Oil','Gas']
    if nfmix>0:
        for i in range(nfmix):
            zflag.append(0)
            zcase.append(fmname[i])
    
    print('Bar Chart Plotting Tool') 
    print('')
    
    #select the Reservoir/Fluid Cases to Plot
    aflag=False
    while aflag==False:
        
        print('  Available Fluid Cases for Plotting')
        
        print('')
        print('0          Brine',end='')
        if zflag[0]>0:
            print('*')
        else:
            print('')
        print('1          Oil',end='')
        if zflag[1]>0:
            print('*')
        else:
            print('')
        print('2          Gas',end='')
        if zflag[2]>0:
            print('*')
        else:
            print('')
        if nfmix>0:
            for i in range(0,nfmix):
                print(str(i+3),'        ',fmname[i],end='')
                if zflag[i+3]>0:
                    print('*')
                else:
                    print('')   
        #set the cases to plot
           
        atext='Enter a Case to choose for Plotting, * to Make the Plot\n:'    
        ans=input(atext)
        if ans=='*':
            aflag=True
        else:
            try:
                choice=int(ans)
                if choice < (nfmix+3):
                    zflag[choice]=int(math.fabs(zflag[choice]-1))
            except:
                print('Please Enter a Case Number')
   
    
    #build the data arrays
    #Build the x array
    numangle =int((thetaend-thetabeg)/thetainc+1)
    for i in range(0,numangle):
        x.append(i*thetainc+thetabeg)
    #Get the labels for the cases
    #make a plot for each shale
    ymin=0.0
    ymax=0.0
    sh=0
    if numsh>1:
        print('Multiple Bounding Shale Cases are Defined\n')
        for i in range(0,numsh):
            print('Shale: ',i,' ',descsh[i])
        ans=input('Enter the Shale Number to use for the Plot')
        try:
            sh=int(ans)
            if sh>(numsh-1):
                sh=numsh-1
        except:
            sh=0
    print('Shale ',sh,' ',descsh[sh],'used for plot')
    plt.figure(figsize=(8,4),dpi=80)            
    for i in range(0,nfmix+3):
        if zflag[i] > 0:
           print(zcase[i])
           jtxt=zcase[i]
           if 'bri' in jtxt.lower():
               col='blue'        
           elif 'oil' in jtxt.lower():
               col='green'
           elif 'gas' in jtxt.lower():
               col='red'
           else:
               col='yellow' 
           del y[0:len(y)]
           for k in range(0,numangle):
               #print(sh,i,k)
               y.append(ava[sh,i,k])
               #print (y[k])
               if y[k]<ymin:
                   ymin=y[k]
               if y[k]>ymax:
                   ymax=y[k]
           plt.plot(x,y,color=col,label=zcase[i])
           
           #plt.plot(x,yplot,color=col,linewidth=1.0,linestyle="-")
    ax=plt.subplot(111)
    plt.ylim(min(ymin*1.1,0),max(0,ymax*1.1))
    ax.legend(loc='center right',bbox_to_anchor=(1.3,0.7))
    ax.yaxis.grid(b=True, which='major', color='black', linestyle='--')
    ax.xaxis.grid(b=True, which='major', color='black', linestyle='--')
    plt.title('Reflection Coefficient Vs Incidence Angle')
    plt.xlabel('Incidence Angle (Degrees)')
    plt.ylabel('Reflection Coefficient')
    plt.show()
        
    #plt.ylim(ymin,ymax)
    #plt.xlim(thetabeg,thetaend)
        
          
    #plotting
    
    #ax.set_ylabel('Reflection Coefficient')
    #ax.set_title('Reflection Coefficient Vs. Angle')
#    ax.set_xticks(ind + width)
#    ax.set_xticklabels(names)
    #ax.yaxis.grid(b=True, which='major', color='b', linestyle='-')
    
    
    
    
    
    
    #plt.show()       
    presskey()
        
    return
 
def writeava(avafile):
    print('AVA Data Output File: ',avafile)
    #select the Reservoir/Fluid Cases to Plot
    zflag=list()
    zcase=list()
    zflag=[0,0,0]
    
    zcase=['Brine','Oil','Gas']
    if nfmix>0:
        for i in range(nfmix):
            zflag.append(0)
            zcase.append(fmname[i])
    aflag=False
    while aflag==False:
        
        print('  Available Fluid Cases for AVA Export')
        
        print('')
        print('0          Brine',end='')
        if zflag[0]>0:
            print('*')
        else:
            print('')
        print('1          Oil',end='')
        if zflag[1]>0:
            print('*')
        else:
            print('')
        print('2          Gas',end='')
        if zflag[2]>0:
            print('*')
        else:
            print('')
        if nfmix>0:
            for i in range(0,nfmix):
                print(str(i+3),'        ',fmname[i],end='')
                if zflag[i+3]>0:
                    print('*')
                else:
                    print('')   
        #set the cases to plot
           
        atext='Enter a Case to choose for Export, * to do Export\n:'    
        ans=input(atext)
        if ans=='*':
            aflag=True
        else:
            try:
                choice=int(ans)
                if choice < (nfmix+3):
                    zflag[choice]=int(math.fabs(zflag[choice]-1))
            except:
                print('Please Enter a Case Number')
  
    print('FLuid Cases for Export')
    for i in range(0,len(zflag)):
        if zflag[i]>0:
            print(i,' ',zcase[i])
    #Choose shale for AVA Output
    sh=0
    if numsh>1:
        print('Multiple Bounding Shale Cases are Defined\n')
        for i in range(0,numsh):
            print('Shale: ',i,' ',descsh[i])
        ans=input('Enter the Shale Number to use for Export')
        try:
            sh=int(ans)
            if sh>(numsh-1):
                sh=numsh-1
        except:
            sh=0
    print('Shale ',sh,' ',descsh[sh],'used for Export')
    #Open the Output File
    try:
        fhand=open(avafile,'w')
    except:
        print(' File: ',rmfile,' cannot be opened')
        presskey()
        
        return 
    #write the data
    numangle =int((thetaend-thetabeg)/thetainc+1)    
    linea='ShaleCase '+descsh[sh]+'\n'
    fhand.write(linea)
    for i in range(0,nfmix+3):
        if zflag[i]>0:
            linea='FluidCase '+zcase[i]+'\n'
            fhand.write(linea)
            linea='Angle(Degrees) Amplitude\n'
            fhand.write(linea)
            print('Exporting Fluid ',zcase[i])
            for j in range(0,numangle):
                theta=thetabeg+j*thetainc
                linea=str(theta)+' '+str(ava[sh,i,j])+'\n'
                fhand.write(linea)
    print('AVA Data Export Completed')    
    fhand.close()
    return
  
  
  
    
# Main Program Loop    
ansmm = ''
#==============================================================================
# testt=gettext('Well Name',20,'')
# print (testt)
#==============================================================================


while ansmm!='q':   
    ansmm = mainmenu()
    if verbose=='yes':
        print ('  Main Menu Choice =',ansmm)
# 1 ********************************************************************
    if ansmm=='1':                  #Load the RESERVOIR ROCK PROPERTIES FILE
        print('Load the RESERVOIR (GASSMANN or OBLIQUE)  MODEL File')
        tlab=''
        rw='r'
        drect=''
        typ='.txt'
        fname=getfname(rw,drect,typ,tlab)
        d=loadava(fname,d)   #returns the directory with all the parameters
        rmfile=fname
        rmfiledesc=d['desc']
        
        ans=True
        if ans==True: #real the fluid properties
            print('Reading the Fluid Properties...')
            flfile=fname
            flfiledesc=d['desc']
            press=d['press']
            temp =d['temp']
            #water
            vw        =d['vw']
            rhow      =d['rhow']
            kfw       =d['kfw']
            #brine
            sal       =d['sal']
            vb        =d['vb']
            rhob      =d['rhob']
            kfb       =d['kfb']
            #oil
            api       =d['api'] 
            gasgravo  =d['gasgravo'] 
            rg        =d['rg']
            vo        =d['vo']
            rhoo      =d['rhoo']
            kfo       =d['kfo']
            #gas
            gasgrav   =d['gasgrav'] 
            vg        =d['vg']
            rhog      =d['rhog']
            kfg       =d['kfg']
            #fluid Mixes
            #delete the values in the lists
            del fmname[0:len(fmname)]
            del fmtype[0:len(fmtype)]
            del fmsat[0:len(fmsat)]
            del fmvp[0:len(fmvp)]
            del fmrho[0:len(fmrho)]
            del fmk[0:len(fmk)]
            # determine nfmix
            kk=0
            nfflag=False
            while nfflag==False:
                try:
                    fm=d['fmix'+str(kk)]
                    kk=kk+1
                except:
                    nfflag=True
            nfmix=kk
            if nfmix>0:
                for i in range(0,nfmix):
                    fmname.append(d['fmname'+str(i)])
                    fmtype.append(d['fmtype'+str(i)])
                    fmsat.append(d['fmsat'+str(i)])
                    fmvp.append(d['fmvp'+str(i)])
                    fmrho.append(d['fmrho'+str(i)])
                    fmk.append(d['fmk'+str(i)])
            print('Read of Fluid Parameters Completed')
        #read the reservoir rock properties
        print('Read the Reservoir Rock Properties')
        rarea    =d['rarea']
        rwell    =d['rwell']
        rtype    =d['rtype']
        rname    =d['rname']
        rfluid   =d['rfluid']
        rdepth   =d['rdepth']
        vpr      =d['vpr']
        vsr      =d['vsr']
        rhor     =d['rhor']
        phir     =d['phir']
        kr       =d['kr']
        mur      =d['mur']
        rhograin =d['rhograin']
        #read the in-situ fluid properties
        print('Read the In-Situ Fluid Properties')
        fnamei =d['fnamei']
        ftypei  =d['ftypei']
        vi     =d['vi']
        rhoi   =d['rhoi']
        kfi    =d['kfi']
        #read the dry rock and mineral modulus
        print('Read the Dry Rock and Mineral Moduli')
        k0     =d['k0']
        kd     =d['kd']
        #read the fluid substitution calculations
        print('Read the Fluid Substitution Cases')
        nfsub  =int(d['nfsub'])
        if nfsub>0:
            del fsname[0:len(fsname)]    
            del fssat[0:len(fssat)] 
            del fstype[0:len(fstype)]    
            del fsvpr[0:len(fsvpr)]    
            del fsvsr[0:len(fsvsr)] 
            del fsrhor[0:len(fsrhor)]    
            del fskr[0:len(fskr)]
            del fsmur[0:len(fsmur)]
            for i in range(0,nfsub):
                fsname.append( d['fsname'+str(i)])
                fstype.append( d['fstype'+str(i)])
                fssat.append(  d['fssat' +str(i)])
                fsvpr.append(  d['fsvpr' +str(i)])
                fskr.append(   d['fskr'  +str(i)])
                fsrhor.append( d['fsrhor'+str(i)])
                fsvsr.append(  d['fsvsr' +str(i)])
                fsmur.append(  d['fsmur' +str(i)])
        #load the bounding shale data
        print('Read the Bounding Shale Properties')
        try:
            numsh  =int(d['numsh'])     
            if numsh>0:
                del descsh[0:len(descsh)]    
                del vpsh[0:len(vpsh)] 
                del vssh[0:len(vssh)]    
                del rhosh[0:len(rhosh)] 
                for i in range(0,numsh):
                    descsh.append(d['descsh'+str(i)])
                    vpsh.append(d['vpsh'+str(i)])
                    vssh.append(d['vssh'+str(i)])
                    rhosh.append(d['rhosh'+str(i)])          
                print('READ THE FLUID CASES SELECTED FOR AVA')
                if nfsub>0:
                    del pcase[0:len(pcase)]    
                    del pflag[0:len(pflag)] 
                    for i in range(0,nfsub):
                        pcase.append(d['pcase'+str(i)])
                        pflag.append(d['pflag'+str(i)])
                #load the ava angle ranges
                print('Read the AVA Angle parameters')
                thetabeg    =d['thetabeg']
                thetaend   =d['thetaend']
                thetainc   =d['thetainc']
        except: 
            print('No Bounding Shale/AVA Parameters Present')  

# 2 ************************************************************************               
    if ansmm=='2':   #Set the Bounding Shale(s) Properties
        print(' Set the Bounding Shale Properties')
        print('')
        dflag=False
        if numsh>0:
            print('Specified Bounding Shales\n')
            for i in range(0,numsh):
                print('Shale ',i,' Desc ',descsh[i])
                print('Vp  ',vpsh[i],' m/s')
                print('Vs  ',vssh[i],' m/s')
                print('Rho ',rhosh[i],' m/s')
            prompt='Do you want to keep these values y(es) or n(o)?'
            compl='Please answer y(es) or n(o)'
            ans=ask_ok(prompt, retries=4, complaint=compl)  
            dflag=ans 
        if dflag==False:
            numsh=0
            #delete any previous shale values in lists
            del descsh[0:len(descsh)]
            del vpsh[0:len(vpsh)]
            del vssh[0:len(vssh)]
            del rhosh[0:len(rhosh)]
            while dflag==False:
                inname='Shale Description'
                maxlen=20
                qtext1=''
                descsh.append(gettext(inname,maxlen,qtext1))
                inname='Shale Density'
                iunit='g/cm^3'
                qtext1=''
                tfmin=1.0
                tfmax=4.0
                rhosh.append(getfloat(inname,iunit,qtext1,tfmin,tfmax))
                inname='Shale P-Velocity'
                iunit='m/s'
                qtext1=''
                tfmin=1000.0
                tfmax=6000.0
                vpsh.append(getfloat(inname,iunit,qtext1,tfmin,tfmax)) 
                inname='Shale S-Velocity'
                iunit='m/s'
                qtext1=''
                tfmin=500.0
                tfmax=6000.0
                vssh.append(getfloat(inname,iunit,qtext1,tfmin,tfmax)) 
                numsh+=1
                prompt='Enter Another Shale (Enter y(es or n(o))'
                compl='Please Enter y(es) or n(o)'
                ans=ask_ok(prompt, retries=4, complaint=compl)
                if ans==False:
                    dflag=True
            
            if numsh>0:
                print('\nSpecified Bounding Shales\n')
                for i in range(0,numsh):
                    print('Shale ',i,' Desc ',descsh[i])
                    print('Vp  ',vpsh[i],' m/s')
                    print('Vs  ',vssh[i],' m/s')
                    print('Rho ',rhosh[i],' m/s')
        presskey()

# 3 **************************************************************************   
    if ansmm=='3':    #Select the Pore Fluids to ModeL
        
        if len(pflag)==0:
            pflag=[0,0,0]    
        if len(pcase)==0:
            pcase=['Brine','Oil','Gas']
            if nfmix>0:
                for i in range(nfmix):
                    pflag.append(0)
                    pcase.append(fmname[i])
        cflag=False
        while cflag==False:            
            print('  Available Fluid Cases for Analysis')            
            print('')
            print('0          Brine',end='')
            if pflag[0]>0:
                print('*')
            else: 
                print('')
            print('1          Oil',end='')
            if pflag[1]>0:
                print('*')
            else:
                print('')
            print('2          Gas',end='')
            if pflag[2]>0:
                print('*')
            else:
                print('')
            if nfmix>0:
                for i in range(0,nfmix):
                    print(str(i+3),'        ',fmname[i],end='')
                    if pflag[i+3]>0:
                        print('*')
                    else:
                        print('')   
            #set the cases to plot
               
            atext='Enter Case to choose for Analysis, * Finished Selection\n:'    
            ans=input(atext)
            if ans=='*':
                cflag=True
            else:
                try:
                    choice=int(ans)
                    if choice < (nfmix+3):
                        pflag[choice]=int(math.fabs(pflag[choice]-1))
                except:
                    print('Please Enter a Case Number')
                    
        print('\nFluid Cases Selected for Analysis')
        for i in range(0,3+nfmix):
            if pflag[i]>0:
                print('Case ',i,'Fluid ',pcase[i])
        presskey()
        
# 4 **************************************************************************
    if ansmm=='4':    #calculate the AVA
        print('\nCalculate Amplitude Variation with Offset\n')
        ans=False
        if thetaend>0.0:
            print('Angle Range:',thetabeg,' ',thetaend,' Angle inc:',thetainc)
            prompt='Is this Angle Range Correct? (Enter y(es or n(o))'  
            compl='Please Enter y(es) or n(o)'
            ans=ask_ok(prompt, retries=4, complaint=compl) 
        if ans==False:
            inname='Beginning Offset Angle'
            qtext1='typically 0.0 (No Default)'
            iunit='Degrees'
            qtext1=''
            tfmin=0.0
            tfmax=89.0
            thetabeg=getfloat(inname,iunit,qtext1,tfmin,tfmax)
            inname='Ending Offset Angle'
            iunit='Degrees'
            qtext1='typically 40.0-50.0, (No Default)'
            tfmin=0.0
            tfmax=90.0
            thetaend=getfloat(inname,iunit,qtext1,tfmin,tfmax) 
            print('Angle Range: ',thetabeg,' ',thetaend,' Angle inc. ',thetainc)
            prompt='Is this Angle Range Correct? (Enter y(es or n(o))'        
            compl='Please Enter y(es) or n(o)'
            ans=ask_ok(prompt, retries=4, complaint=compl)  
        if ans==True:
            numangles=int((thetaend-thetabeg)/thetainc + 1.0)
            for i in range(0,numsh):
                print('Shale Case ',i,' ',descsh[i])
                for j in range(0,len(pflag)):               
                    if pflag[j]>0:
                        print('Fluid Case ',j,' ',pcase[j])
                        avacase.append(pcase[j])
                        vp1=vpsh[i]
                        vs1=vssh[i]
                        rho1=rhosh[i]
                        vp2=fsvpr[j]
                        vs2=fsvsr[j]
                        rho2=fsrhor[j]
                        #print('ava inputs ',vp1,vs1,rho1,vp2,vs2,rho2)
                        for k in range(0,numangles):
                            theta1=k*thetainc
                            ava[i,j,k]=zoepj(vp1,vs1,rho1,vp2,vs2,rho2,
                                theta1)
                            #print(descsh[i],pcase[j],k,theta1,ava[i,j,k])        
            print('Amplitude Coefficient Calculations Completed')
        else:
            print('Angle Range for Calculation was not Specified')
        presskey()
        
        
        
    
    if ansmm=='p':
        printava()  
    if ansmm=='s':
        tlab='Enter a Reservoir/AVA Properties Model Output File Name'
        rw='w'
        drect=''
        typ='.txt'
        fname=getfname(rw,drect,typ,tlab)
        saveava(fname)
  
    if ansmm=='e':
        print('AVA Data Export\n')
        tlab='Enter an Output File Name for the AVA Data Export\n'
        rw='w'
        drect=''
        typ='.dat'
        fname=getfname(rw,drect,typ,tlab)
        writeava(fname)
        presskey()
    if ansmm=='*':
        plotavo()
    if ansmm=='x':
        print("EXIT: Enter 'y' to exit or anything else to continue" )
        print("Make sure you have saved your data to file!")
        ans=input()
        if ans=='y':
            ansmm='q'

print('  QUITTING PROGRAM by user request')

        



