
# coding: utf-8
# PREMIUM BLEND 8.0
# J.Coggins, Coggins Geosciences, Inc.
# Converted to Python3.5 on 16sep16
# Functions were tested by comparison to values in the fluid properties and
#  Gassmann substitution sections of Mavko,'The Rock Physics Handbook'
# inputs: press in MPa, Temp in Degrees C
# outputs: vw,rhow,kw
# 
#
#Revision Notes
# 26aug16  Tested functions and began program
# 26aug16  Completed mainmenu(),setp(),cbrine(),coil(),cgas(),
#           printfp()
# 27aug17  fpsave(),fpload(),exit functions
#Version 6.0 completed!
# Version 7 will include building fluid mixes for use in Gassmannager
#02sep16   added fmix(), edited save() for fluid mixes
#03sep16   completed prit function for fluids including mixes
#04sep16  Version 8 completed
#05sep16  replaces lines with the presskey() function

import math
import numpy as np


#  set verbose mode for debuggins
verbose='no'

#global variable definitions
press      = 0.0                #pressure in MPa
temp       = 0.0                #temperature in Celsius Degrees
vw         = 0.0                #velocity of water in meters/sec                   
rhow       = 0.0                #density of water in grams/cm^3
kfw        = 0.0                #fluid modulus of water in GPa
vb         = 0.0                #velocity of brine in meters/sec
rhob       = 0.0                #density of brine in grams/cm^3
kfb        = 0.0                #fluid modulus of brine in GPa
vo         = 0.0                #velocity of oil in meters/sec
rhoo       = 0.0                #density of oil in grams/cm^3
kfo        = 0.0                #fluid modulus of oil in GPa
vg         = 0.0                #velocity of gas in meters/sec
rhog       = 0.0                #density of gas in grams/cm^3
kfg        = 0.0                #fluid modulus of gs in GPa
sal        = 0.0                #salinity of brine in volume fraction
gasgrav    = 0.0                #gas gravity (unitless)
api        = 0.0                #api of oil (unitless)
gasgravo   = 0.0                #gas gravity in oil case (unitless)
rg         = 0.0                #gas oil ratio at STP (unitless)
fpfile     = ''                 #fluid property file name
fpfiledesc = ''                 #fluid file description
a=0.0                           #general use float value
b=0.0                           #general use float value
c=0.0                           #general use float value
d=0.0                           #general use float value
e=0.0                           #general use float value
f=0.0                           #general use float value
g=0.0                           #general use float value
h=0.0                           #general use float value
atext=''
btext=''
#variable for fluid mixes
nfmix   = 0   #integer number of existing fluid mixes
fmname =list()      #text -list of fluid mix names
fmsat  =list()      #float
fmtype =list()      #text
fmvp   =list()      #float
fmk    =list()      #float
fmrho  =list()      #float

#define constants for h2oprop
w=np.ndarray((5,4))
#call h2init once to initialize the constants

w[0][0]=1402.85
w[1][0]=4.871
w[2][0]=-0.04783
w[3][0]=1.487e-4
w[4][0]=-2.197e-7
w[0][1]=1.524
w[1][1]=-0.0111
w[2][1]=2.747e-4
w[3][1]=-6.503e-7
w[4][1]=7.987e-10
w[0][2]=3.437e-3
w[1][2]=1.739e-4
w[2][2]=-2.135e-6
w[3][2]=-1.455e-8
w[4][2]=5.230e-11
w[0][3]=-1.197e-5
w[1][3]=-1.628e-6
w[2][3]=1.237e-8
w[3][3]=1.327e-10
w[4][3]=-4.614e-13
if verbose == 'yes':
    for j in range(0,4):
        for i in range(0,5):
            print (i,j,w[i][j])
            
            
            
def presskey():
	answer=input('Press ENTER to Continue\n:')
	return True
        
def ask_ok(prompt, retries=4, complaint='Please Enter y(es) or n(o)'):
    while True:
        ok = input(prompt)
        if ok in ('y', 'ye', 'yes','Y','Yes','YES'):
            return True
        if ok in ('n', 'no', 'nop', 'nope','N','No','NO'):
            return False
        retries = retries - 1
        if retries < 0:
            return False
        print (complaint)  
        
        
def gettext(inname,maxlen,qtext1):
    """
    gettext returns a text string from the user
    inname is the name of the input property/variable (40 char max)'
    maxlen is max length of input string (def = 20)
    qtext1 is the secondary text  - default is ''  (40 char max)
    
    """
    inn=''
    if maxlen==0:
        maxl=20
    else:
        maxl=maxlen
    qt1=''
    qt2=''
    ds1=''
    qc='q'
    rc='r'
    # apply the default values for inputs
    if inname=='':
        print('Error: Input name inname cannot be null')
        return ''
    inn=inname[0:min(len(inname),40)]
    qt1=qtext1[0:min(len(qtext1),40)]    
    intext=''
    rflag=0
    while rflag==0:
        print(' Enter Text: ',inn.upper(),'(',maxl,' char. max):' ) 
        if qt1!='':
            print('',qt1)
        ans=''
        while ans=='':
            ans=input()
        intext=ans[0:min(len(ans),maxl)]
        print(inn,' = ',intext)
        ans=input('Enter y to accept,r to redo, q to quit: ') 
        print(' ')
        if ans=='r':
            rflag=0
        elif ans=='q':
            rflag=1
            print('Quit by user request')
            presskey()
            return 'q'
        elif ans =='y':
            rflag=1
        else:
            rflag=0
    return intext
    
def getfloat(inname,iunit,qtext1,tfmin,tfmax):
    """
    getfloat() returns a floating point value from the user
    inname is the name of the input property/variable (20 char max)
    iunit  is the units associated with value (10 char max) default-unitles
    qtext1 is the secondary text  - default is '' 
    tfmin is the minimum value accepted as text -default = 0.0
    tfmax is the maximum value accepted as text -default = 9.9e99
    """
    inn=''
    qt1=''
    iu=''
    fmin=0.0
    fmax=0.0
    # apply the default values for inputs
    if inname=='':
        print('Error: Input name inname cannot be null')
        return 0.0
    inn=inname[0:min(len(inname),20)]
    iu = iunit[0:min(len(iunit),10)]
    qt1=qtext1[0:min(len(qtext1),40)]
    if tfmin=='':
        fmin=0.0
    else:
        try:
            fmin=float(tfmin)
        except:
            fmin=0.0
    if tfmax=='':
        fmax=9.9e99
    else:
        try:
            fmax=float(tfmax)
        except:
            fmax=9.9e99
    infloat=0.0
    inval=0.0
    rflag=0
    while rflag==0:
        print(' Enter Value: ',inn.upper(),iu,' min=',fmin,' max=',fmax ) 
        if qt1!='':
            print('',qt1)
        ans=''
        while ans=='':
            ans=input('\n:')
        try:
            inval=float(ans)
        except:
            print('ERROR: Input Value is not in Floating Point format')
            print('Value set to 0.0')
            inval=0.0
        if inval<fmin:
            print('ERROR: Input is Greater that acceptable maximum')
            print('input value set to: ',fmin)
            infloat=fmin
        elif inval>fmax:
            print('ERROR: Input is less that acceptable minimum')
            print('input value set to: ',fmax)
            infloat=fmax
        else:
            infloat=inval
        print(inn,' = ',infloat,' ',iu)
        ans=input('Enter y to accept,r to redo, q to quit: ') 
        print('')
        if ans=='r':
            rflag=0
        elif ans=='q':
            rflag=1
            print('Quit by user request')
            presskey()
            return 0.0
        elif ans =='y':
            rflag=1
        else:
            rflag=0
    return infloat        
        
    
#calculate water properties


def h2oprop(press,temp):
    # Calculate water velocity
    #Pressure in MPa, Temperature in Deg C
    #Output v in m/s, output density in g/cm**3, k in GPa
    #Equations from Mavko, The Rock Physics Handbook, pp 339-346.
    # attributed to Batzle and Wang,1992, Seismic Properties of Pore Fluids,
    #  Gophysics,vol 57, p 1396-1408
    vw=0.0
    for i in range(0,5):
        for j in range(0,4):
            vw=vw+w[i][j]*math.pow(press,j)*math.pow(temp,i)
   #calculate water density
    t=temp
    p=press
    rhow=0.0
    c1=(1.0e-6)*(-80.0*t-3.3*t*t+0.00175*t*t*t)
    c2=(1.0e-6)
    c2=c2*(489.0*p-2.0*t*p+0.016*t*t*p-(1.3e-5)*t*t*t*p-0.333*p*p-0.002*t*p*p)
    rhow=1.0+c1+c2
     #Calculate water fluid modulus
    kfw=0.0
    kfw=rhow*vw*vw/(1.0e6)  #divide by 1.0e6 to convert to GPa
    return vw,rhow,kfw
    
#Calculate brine properties

vb=0.0
rhob=0.0
def brprop(press,temp,vw,rhow,sal):
    #Pressure in MPa, Temperature in Deg C, salinity is in fractions of one (
    #   parts per million divided by 10**6)
    #Output v in m/s, output density in g/cm**3
    #Equations from Mavko, The Rock Physics Handbook, pp 339-346.
    # attributed to Batzle and Wang,1992, Seismic Properties of Pore Fluids,
    #  Gophysics,vol 57, p 1396-1408
    #Calculate brine velocity
    vb=0.0
    p=press
    t=temp
    s=sal
    c1=1170.0-9.6*t+0.055*t*t-(8.5e-5)*t*t*t+2.6*p-0.0029*t*p-0.0476*p*p
    c2=780.0-10.0*p+0.16*p*p
    vb=vw + s*c1+math.pow(s,1.5)*c2-1820.0*s*s
    #calculate brine density
    rhob=0.0
    c1=300.0*p-2400.0*p*s+t*(80.0+3.0*t-3300.0*s-13.0*p+47.0*p*s)
    #print ('c1=',c1)
    rhob=rhow+s*(0.668+0.44*s+(1.0e-6)*c1)
    #Calculate brine fluid modulus
    kfb=0.0
    kfb=rhob*vb*vb/(1.0e6)  #divide by 1.0e6 to convert to GPa
    return vb,rhob,kfb

#Calculate Gas Properties

def gasprop(press,temp,gasgrav):
     #Pressure in MPa, Temperature in Deg C, gas gravity is unitless.
    #Output v in m/s, output density in g/cm**3, output K in GPa
    #Equations from Mavko, The Rock Physics Handbook, pp 339-346.
    # attributed to Batzle and Wang,1992, Seismic Properties of Pore Fluids,
    #  Gophysics,vol 57, p 1396-1408
    rhog=0.0
    g=gasgrav
    pr=press/(4.892-0.4048*g)
    ta=(temp+273.15)
    tr=ta/(94.72+170.75*g)
    #density calculation
    d=math.exp(-(0.45+8.0*math.pow((0.56-1.0/tr),2))*math.pow(pr,1.2)/tr)
    c=0.109*math.pow((3.85-tr),2)
    b=0.642*tr-0.007*math.pow(tr,4)-0.52
    a=0.03+0.00527*math.pow((3.5-tr),3)
    r=8.31441
    z=a*pr+b+c*d
    #print a,b,c,d,r,z
    rhog=28.8*g*press/(z*r*ta)
    #calculate modulus  
    kfg=0.0       
    d=math.exp(-(0.45+8.0*math.pow((0.56-1.0/tr),2))*math.pow(pr,1.2)/tr)
    c=0.109*math.pow((3.85-tr),2)
    b=0.642*tr-0.007*math.pow(tr,4)-0.52
    a=0.03+0.00527*math.pow((3.5-tr),3)
    r=8.31441
    z=a*pr+b+c*d
    m=1.2*(-(0.45+8.0*math.pow((0.56-1.0/tr),2))*math.pow(pr,0.2)/tr)
    f=c*d*m+a
    y=0.85+5.6/(pr+2)+27.1/math.pow((pr+3.5),2)-8.7*math.exp(-0.65*(pr+1))
    kfg=press*y/(1-pr*f/z)/1000.0  # convert to GPa
    #Calculate Vg
    vg=0.0
    vg=math.sqrt(kfg/rhog*1.0e6)   #1.0E6 converts units to m/s

    return vg,rhog,kfg
    
# calculate oil properties
    
def oilprop(press,temp,api,gasgrav,rg):
    #Pressure in MPa, Temperature in Deg C, gas gravity is unitless,
    #  api is unitless, rg is the unitless volume ratio of gas to oil at STP
    #Output v in m/s, output density in g/cm**3, output K in GPa
    #Equations from Mavko, The Rock Physics Handbook, pp 339-346.
    # attributed to Batzle and Wang,1992, Seismic Properties of Pore Fluids,
    #  Gophysics,vol 57, p 1396-1408.

    g=gasgrav
    t=temp
    p=press
    po=0.0
    bo=0.0
    pr=0.0
    pg=0.0
    pp=0.0
    rhoo=0.0
    vo=0.0
    kfo=0.0
    #density calculation
    po=141.5/(api+131.5)
    bo=0.972+0.00038*math.pow((2.4*rg*math.pow((g/po),0.5)+t+17.8),1.175)
    pr=po/bo*1.0/(1.0+0.001*rg)
    pg=(po+0.0012*g*rg)/bo
    pp=pg+(0.00277*p-(1.71e-7)*p*p*p)*math.pow(pg-1.15,2)+(3.49e-4)*p
    rhoo=pp/(0.972+3.81e-4*math.pow(t+17.78,1.175))
    
    vo=0.0
    #velocity calculation    
    vo=2096.0*math.pow(pr/(2.6-pr),0.5)-3.7*t+4.64*p
    vo=vo+0.0115*(4.12*math.pow(1.08/pr-1.0,0.5)-1.0)*t*p
    
    #Calculate the oil modulus
    kfo=0.0
    kfo=rhoo*vo*vo/(1.0e6)  #divide by 1.0e6 to convert to GPa
    return vo,rhoo,kfo

# Main Menu Function

def mainmenu():
    ansmm=''
    while ansmm!='q':
#        for i in range(0,50):
#            print('')
        print('/////////////////////////////////////////////////////////')        
        print('/           COGGINS GEOTOOLS:  PREMIUM BLEND            /')
        print('/                                                       /')
        print('/                   By J. Coggins                       /')
        print('/              Coggins Geosciences, Inc.                /')
        print('/                                                       /')   
        print('/    PREMIUM BLEND generates fluid properties for       /')  
        print('/    use in fluid substitution. Every effort is made    /')
        print('/    to ensure the integrity of the results, but the    /')       
        print('/    end user is solely responsible for the quality     /')
        print('/    and accuracy of the results.                       /')
        print('/                                                       /')
        print('/                Enter a Menu Choice:                   /')
        print('/    1    Set PRESSURE and TEMPERATURE values           /')
        print('/    2    Calculate BRINE Properties                    /')
        print('/    3    Calculate OIL   Properties                    /')
        print('/    4    Calculate GAS   Properties                    /')
        print('/    m    Calculate a FLUID MIXTURE                     /')
        print('/    p    PRINT the current FLUID PROPERIES             /')
        print('/    s    SAVE the current FLUID PROPERTIES to a FILE   /')
        print('/    l    LOAD the FLUID PROPERTIES from a FILE         /')
        print('/    x    EXIT PREMIUM BLEND                           /')
        print('/////////////////////////////////////////////////////////')       
        ansmm=input(' ENTER CHOICE: ')
        return ansmm
    
def setpt():
    p=0.0
    t=0.0
    p=getfloat('Fluid Pressure','MPa','','0','1000')
    t=getfloat('Temperature','Deg. C','','0','500')
    
    print('Pressure    = ',p,' MPa')
    print('Temperature = ',t, ' Deg. C')
    print('')
    
    presskey()
    return p,t    

def cbrine():
    print('')
    print(' Calculate Brine Properties')
    print('')
    print(' Pressure   = ',press,' MPa')
    print(' Temp.      = ',temp,' Deg. C')
    print(' vw         = ',vw,' m/s')
    print(' rhow       = ',rhow,' g/cm^3')
    print(' kfw        = ',kfw,' GPa')
    sal=getfloat('Salinity','PPM','','0','2000000')
    print('Salinity    = ',sal,' PPM')
    sal=sal/1.0e6
    print('Salinity    = ',sal,' Volume Fraction')  
    print('')
    vb,rhob,kfb=brprop(press,temp,vw,rhow,sal)
    print('vb          = ',vb,' m/s')
    print('rhob        = ',rhob,' g/cm^3')
    print('kfb         = ',kfb,' GPa')
    presskey()
    return vb,rhob,kfb,sal    
    
def coil():
    print('')
    print(' Calculate Oil Properties')
    print('')
    print(' Pressure   = ',press,' MPa')
    print(' Temp.      = ',temp,' Deg. C')
    api=getfloat('API','Unitless','','0','500')
    print(' API        = ',api)
    gasgravo=getfloat('Gas Gravity(in Oil)','Unitless','','0','40')
    print(' gasgravo   = ',gasgravo)
    rg=getfloat('Gas Oil Ratio','Volume Ratio(STP)','','0','1e6')
    print(' rg         = ',rg)
    print('')
    vo,rhoo,kfo=oilprop(press,temp,api,gasgravo,rg)
    print('vo          = ',vo,' m/s')
    print('rhoo        = ',rhoo,' g/cm^3')
    print('kfo         = ',kfo,' GPa')
    presskey()
    return vo,rhoo,kfo,api,gasgravo,rg        

def cgas():
    print('')
    print(' Calculate Gas Properties')
    print('')
    print(' Pressure   = ',press,' MPa')
    print(' Temp.      = ',temp,' Deg. C')
    gasgrav=getfloat('Gas Gravity','Unitless','','0','50')
    print(' gasgrav    = ',gasgrav)
    print('')
    vg,rhog,kfg=gasprop(press,temp,gasgrav)
    print(' vg         = ',vg,' m/s')
    print(' rhog       = ',rhog,' g/cm^3')
    print(' kfg        = ',kfg,' GPa')
    presskey()
    return vg,rhog,kfg,gasgrav       

def fmix():
    """
    Fmix() creates a fluid by mixing brine and gas or water
    swat is the fractional water (brine) saturation (float)
    ftype is the fluid - 'gas',oil
    """
    print('')
    print(' Calculate a FLUID MIXTURE')
    print('')
    print(' Input Fluids must be defined prior to mixing')
    print('')
    print(' Existing Fluid Definitions')
    if vb>0.0:
        print('Brine: Salinity=',sal)
    if vo>0.0:
        print('Oil:  API=',api,'Gasgrav=',gasgravo,'GOR=',rg)
    if vg>0.0:
        print('Gas:  Gas Gravity=',gasgrav)
    if nfmix>0:
        for i in range(0,nfmix):
            print('Mix',i,fmname[i],': Sw=',fmsat[i],' HC Type=',fmtype[i])
    mixname=gettext('Name of Fluid Mixture',20,' (No Spaces in Name)')
    print(' Fluid MIxture Name = ',mixname)
    swat=getfloat('Water Saturation','Fraction','','0.0','1.0')   
    print(' Sw                 = ',swat)
    ftype=gettext('Fluid Name (gas or oil)',20,'')   
    print(' Fluid Type         = ',ftype)
    if ftype=='gas':
        rhof=rhog
        kf=kfg
    elif ftype=='oil': 
        rhof=rhoo
        kf=kfo
    k=1.0/(swat/kfb+(1-swat)/kf)
    #print(k,kfb,kf)
    rho=swat*rhob+(1.0-swat)*rhof
    vp=math.sqrt(k/rho*1.0e6)  #convert to m/s
    nummix=nfmix+1
    #print (nfmix,nummix)
    return nummix,mixname,swat,ftype,vp,rho,k       

    
    
def printfp():
    print('')
    print ('  CURRENT FLUID PROPERTIES')
    if fpfile!='':
        print('Current Fluid Property File:',fpfile)
    if fpfiledesc!='':
        print('Current File Description:',fpfiledesc)
    print('')
    print(' Pressure   = ',press,' MPa')
    print(' Temp.      = ',temp,' Deg. C')
    print('')
    print('WATER')
    print(' vw         = ',vw,' m/s')
    print(' rhow       = ',rhow,' g/cm^3')
    print(' kfw        = ',kfw,' GPa')
    print('')
    print('BRINE')
    print(' Salinity   = ',sal,' Volume Fraction')   
    print(' vb         = ',vb,' m/s')
    print(' rhob       = ',rhob,' g/cm^3')
    print(' kfb        = ',kfb,' GPa')
    print('')
    print('OIL')
    print(' API        = ',api)   
    print(' gasgravo   = ',gasgravo)    
    print(' rg         = ',rg)      
    print(' vo         = ',vo,' m/s')
    print(' rhoo       = ',rhoo,' g/cm^3')
    print(' kfo        = ',kfo,' GPa')
    print('')
    print('GAS')
    print(' gasgrav    = ',gasgrav)   
    print(' vg         = ',vg,' m/s')
    print(' rhog       = ',rhog,' g/cm^3')
    print(' kfg        = ',kfg,' GPa')
    print('')
    if nfmix>0:
        print(' Mixed Fluid Cases')
        if nfmix > 0:
            for i in range(0,nfmix):
                print('Mix',i,fmname[i],': Sw=',fmsat[i],' HC Type=',
                fmtype[i])
                print(' vp         = ',fmvp[i],' m/s')
                print(' rho        = ',fmrho[i],' g/cm^3')
                print(' kf         = ',fmk[i],' GPa')
                      
    presskey()  
 
def savefp(fname):
    print('')
    print(' Save Fluid Properties File')
    print('')
        
    fpfile=fname
    
    print(' Fluid File    = ',fpfile)
    print('')
    print('Enter a Fluid File Description (40 char max, NO SPACES)')
    ans=input(':')
    
    if len(ans)>40:
        fpfiledesc=ans[0:40]
    elif len(ans)==0:
        fpfiledesc='none' 
    else:
        fpfiledesc=ans
    try:
        fhand=open(fpfile,'w')
    except:
        print(' File: ',fpfile,' cannot be opened')
        presskey()
        fpfile=''
        fpfiledesc=''
        return
    #write the file description
    line0='desc '+fpfiledesc+'\n'
    fhand.write(line0)
    #write the fluid properties
    line1='press '+str(press)+' MPa\n'
    fhand.write(line1)
    line2='temp '+str(temp)+' DegC\n'
    fhand.write(line2)
    #
    line3='WATER PROPERTIES\n'
    fhand.write(line3)
    line4='vw '+str(vw)+' m/s\n'
    fhand.write(line4) 
    line5='rhow '+str(rhow)+' g/cm^3\n'
    fhand.write(line5)
    line6='kfw '+str(kfw)+' GPa\n'
    fhand.write(line6)
    #
    line7='BRINE PROPERTIES\n'
    fhand.write(line7)
    line8='sal '+str(sal)+' fraction\n'
    fhand.write(line8) 
    line9='vb '+str(vb)+' m/s\n'
    fhand.write(line9) 
    line10='rhob '+str(rhob)+' g/cm^3\n'
    fhand.write(line10)
    line11='kfb '+str(kfb)+' GPa\n'
    fhand.write(line11)
    #
    line12='OIL PROPERTIES\n'
    fhand.write(line12)
    line13='api '+str(api)+' unitless\n'
    fhand.write(line13) 
    line14='gasgravo '+str(gasgravo)+' unitless\n'
    fhand.write(line14) 
    line15='rg '+str(rg)+' unitless\n'
    fhand.write(line15) 
    line16='vo '+str(vo)+' m/s\n'
    fhand.write(line16) 
    line17='rhoo '+str(rhoo)+' g/cm^3\n'
    fhand.write(line17)
    line18='kfo '+str(kfo)+' GPa\n'
    fhand.write(line18)
    #
    line19='GAS PROPERTIES\n'
    fhand.write(line19)
    line20='gasgrav '+str(gasgrav)+' unitless\n'
    fhand.write(line20) 
    line21='vg '+str(vg)+' m/s\n'
    fhand.write(line21) 
    line22='rhog '+str(rhog)+' g/cm^3\n'
    fhand.write(line22)
    line23='kfg '+str(kfg)+' GPa\n'
    fhand.write(line23)
    #
    # write the fluid mixture data
    line=list()
    if nfmix>0:
        linea='FLUID MIX PROPERTIES\n'
        fhand.write(linea)
        for i in range(0,nfmix):
            line.append('fmix'+str(i)+' '+str(i)+'\n')
            line.append('fmname'+str(i)+' '+str(fmname[i])+'\n')
            line.append('fmtype'+str(i)+' '+str(fmtype[i])+'\n')
            line.append('fmsat'+str(i)+' '+str(fmsat[i])+' unitless\n')
            line.append('fmvp'+str(i)+' '+str(fmvp[i])+' m/s\n')
            line.append('fmrho'+str(i)+' '+str(fmrho[i])+' g/cm^3\n')
            line.append('fmk'+str(i)+' '+str(fmk[i])+' GPa\n')
            for j in range(0,7):
                fhand.write(line[i*7+j])

    #Write of fluid properties completed
    fhand.close()    
    print('Save File Completed')
    presskey()    
    return  fpfiledesc       
      

def loadfp(fname,fltype):
    
    #fltype is 'desc' (description)'water','brine','oil' or 'gas'
    #print('')
    print(' Loading Fluid Properties for: ',fltype)
    #print('')
   
    fpfile=fname
     
    #print(' Input Fluid File    = ',fpfile)
    
    try:
        fhand=open(fpfile)
    except:
        print(' Input File: ',fpfile,' cannot be opened')
        presskey()
        fpfile=''
        return 0,0,0,0,0,0,0,0,
        
    #Read the Fluid Properties file
    d=dict()
    count=0
    for line in fhand:
#        print (line,end='')
        a=line.split()
        t=a[0]
        val=a[1]
        try:
            d[t]=float(val)
        except:
            d[t]=val
        #print(count,t,d[t]) 
        count=count+1
    #print (d)
    print('')   
    if fltype=='desc':
        return d['desc'],0,0,0,0,0,0,0
    if fltype=='water':
        return d['press'],d['temp'],d['vw'],d['rhow'],d['kfw'],0,0,0
    if fltype=='brine':
        return d['press'],d['temp'],d['vb'],d['rhob'],d['kfb'],d['sal'],0,0
    if fltype=='oil':
        return (d['press'],d['temp'],d['vo'],d['rhoo'],d['kfo'],d['api'],
        d['gasgravo'],d['rg']) 
    if fltype=='gas':
        return d['press'],d['temp'],d['vg'],d['rhog'],d['kfg'],d['gasgrav'],0,0
    if fltype=='nfmix':
        zflag=0
        ii=0
        while zflag==0:
            try:
                zzz=d['fmix'+str(ii)]
                ii=ii+1
            except:
                nfm=ii
                zflag=1
        #print('zflag=',zflag,'nfmix=',nfm)
        return nfm,0,0,0,0,0,0,0
    if nfmix>0:
        #print('Entered nfmix>0 loop in load')
        for i in range(0,nfmix):
            if fltype=='fmix'+str(i):
                print(fltype,i,'fmvp0=',d['fmvp0'],d['fmvp'+str(0)])
                return (d['fmix'+str(i)],d['fmname'+str(i)],d['fmtype'+str(i)],
                d['fmsat'+str(i)],d['fmvp'+str(i)],
                d['fmrho'+str(i)],d['fmk'+str(i)],0.0)
    
def getfname(rw,drect,ftype,tlab):
    #get a file name for read or write
    # and verify file existence   no default
    # rw is 'r' or 'w' for read or write, default is read
    # drect is a directory string (must end with / ) default is none
    # ftype is the file type to use / enforce  default is '.txt'
    #   is 
    print(' ',tlab)
    print('File type ',ftype,' will be appended to name if missing')
    if rw=='r':
        print(' Input file must already exist, or a null name is returned')
    
    ans=''
    fname=''
    while ans=='':
        ans=input('File Name: ')
    if ans[len(ans)-4:len(ans)-1]==ftype:
        fname=drect+ans
    else:
        fname=drect+ans+ftype
    try:
        fhand=open(fname,'r')
    except:
        if rw=='r':
            print(' Input File: ',fname,' cannot be opened')
            presskey()
            fname=''
            return fname
        elif rw=='w':
            return fname
    if rw=='w':
        print(' Output file ',fname,' exists')
        i=ask_ok('Replace Existing File? Enter y(es) or n(o)\n:')
        if i==False:
            fname=''
            fhand.close()
            return fname 
    fhand.close()
    return fname
    
    




# Main Program Loop    
ansmm = ''
zzz=0
while ansmm!='q':   
    ansmm = mainmenu()
    if verbose=='yes':
        print ('  Main Menu Choice =',ansmm)
    if ansmm=='1':
        press,temp=setpt()
    if ansmm=='2':
        vw,rhow,kfw=h2oprop(press,temp)
        vb,rhob,kfb,sal=cbrine()
    if ansmm=='3':
        vo,rhoo,kfo,api,gasgravo,rg=coil()
    if ansmm=='4':
        vg,rhog,kfg,gasgrav=cgas()
    if ansmm=='m':
        nummix,mixname,swat,ftype,vp,rho,k=fmix() 
        print('nummix=',nummix)
        if ftype!='null':
            nfmix=nummix
            fmname.append(mixname)
            fmsat.append(swat)
            fmtype.append(ftype)
            fmvp.append(vp)
            fmrho.append(rho)
            fmk.append(k)
            print('Fluid Mix:',nfmix-1,' ',fmname[nfmix-1])
            presskey()
    if ansmm=='p':
        printfp()
    if ansmm=='s':
        tlab='Enter a Fluid Properties Output File Name'
        rw='w'
        drect=''
        typ='.txt'
        fname=getfname(rw,drect,typ,tlab)
        fpfiledesc=savefp(fname)
    if ansmm=='l':
        tlab='Enter a Fluid Properties Input File Name'
        rw='r'
        drect=''
        typ='.txt'
        fname=getfname(rw,drect,typ,tlab)
        print('Input Fluid File: ',fname)
        ftype='desc'
        fpfiledesc,d,e,f,g,a,b,c = loadfp(fname,ftype)
        #print('File Description: ',fpfiledesc)
        ftype='water'
        press,temp,vw,rhow,kfw,a,b,c = loadfp(fname,ftype)
        print('water',press,temp,vw,rhow,kfw)
        ftype='brine'
        press,temp,vb,rhob,kfb,sal,a,b = loadfp(fname,ftype)
        print('brine',press,temp,vb,rhob,kfb,sal)
        ftype='oil'
        press,temp,vo,rhoo,kfo,api,gasgravo,rg = loadfp(fname,ftype)
        print('oil',press,temp,vo,rhoo,kfo,api,gasgravo,rg)
        ftype='gas'
        press,temp,vg,rhog,kfg,gasgrav,a,b = loadfp(fname,ftype)
        print('gas',press,temp,vg,rhog,kfg,gasgrav)
        ftype='nfmix'
        nn,a,b,c,d,e,f,g=loadfp(fname,ftype)
        nfmix=int(nn)
        print('nfmix=',nfmix)
        if nfmix>0:
            for i in range(0,nfmix):
                ftype='fmix'+str(i)
                zzz,atext,btext,a,b,c,d,e=loadfp(fname,ftype)
                fmname.append(atext)
                fmtype.append(btext)
                fmsat.append(a)
                fmvp.append(b)
                fmrho.append(c)
                fmk.append(d)
                print(zzz,atext,btext,a,b,c)
                
        presskey()
    if ansmm=='x':
        ans=ask_ok('Are You Sure you want to EXIT? Enter y(es) or n(o)\n:')
        if ans==True:
            ansmm='q'

print('  QUITTING PROGRAM by user request')

